/* $Header: /u/sprint2/christos/astr/RCS/aiws.hft.c,v 1.1 89/07/30 01:42:52 christos Exp Locker: christos $ */
/* 
 * hft.c:  Open a `high function terminal' in monitor mode
 * 	   and provide support for it
 *
 * Routines:
 * 	int 			hft_open()	
 *	int 			hft_key()
 *	struct hflocator * 	hft_mouse()
 * 	void 			hft_close()
 * 	void 			hft_wait()
 *
 * $Log:	aiws.hft.c,v $
 * Revision 1.1  89/07/30  01:42:52  christos
 * Initial revision
 * 
 */
#ifndef lint
static char rcsid[] = "$Id: aiws.hft.c,v 1.1 89/07/30 01:42:52 christos Exp Locker: christos $";
#endif /* lint */
 
#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/hwdbus.h>
#include <sys/hft.h>
#include "util.h"
 

private int fd_bus;
private int fd_hft;
private int hft_waitflag = 1;
private	int hft_granted;

/*
 * set monitor signals
 */
private struct hfsmon hft_set_signals = { HFSINGLE };
 
/*
 * structure to write to the hft to enter monitor mode
 */
private struct hfprotocol hft_monitor_mode =  
{
   { HFINTROESC, HFINTROLBR, HFINTROEX, 0, 0, 0, sizeof(struct hfprotocol)-3,
     HFMOMPROH, HFMOMPROL },
   2, /* manual says 4, but is WRONG! */
   0,
   HFXLATKBD, HFLOCATOR, /* translate keystrokes, enable locator */
};
 
/*
 * Set all the bits on the key map on
 */
private int hft_key_map[32] =
{
   -1, -1, -1, -1, -1, -1, -1, -1, 
   -1, -1, -1, -1, -1, -1, -1, -1,
   -1, -1, -1, -1, -1, -1, -1, -1,
   -1, -1, -1, -1, -1, -1, -1, -1,
};
   
/*
 * set keyboard handling
 */
private struct hfbuf hft_set_keys = { (char *) hft_key_map, 128 };
   
/*
 * What to write to the hft to request the screen
 */
private struct hfmomscreq hft_scn_request =   
{
   { HFINTROESC, HFINTROLBR, HFINTROEX, 0, 0, 0, sizeof(struct hfintro)-3, 
     HFMOMREQH,	HFMOMREQL },
   0,		/* unused */
   0,		/* unused */
   0, 0,	/* unused */
   0, 0, 0, 0	/* unused */
};
 
/*
 * what to write to the hft to release the screen
 */
private struct hfmomscrel hft_scn_release = 
{
   { HFINTROESC, HFINTROLBR, HFINTROEX,
   0, 0, 0, sizeof(struct hfmomscrel)-3,
   HFMOMRELH,	
   HFMOMRELL
   },
};
 
/* hft_request():
 *	request use of screen, wait for it to be granted
 */
private int
hft_request(hft)
{
    hft_granted = 0;
    if (write(hft, &hft_scn_request, sizeof(hft_scn_request)) != 
        sizeof(hft_scn_request)) {
	(void) fprintf(stderr, "hft_request: Write failed on `/dev/hft' (%s)\n",
	    PERROR);
	return(-1);
    }

    while (!hft_granted) 
        pause();
    return(0);
   
} /* end hft_request */
   
/* hft_grant():
 *	accept "we can use the screen now" signal
 */
private int
hft_grant()
{ 
    (void) signal(SIGGRANT, hft_grant);
    hft_granted = 1;
} /* end hft_grant */
 
/* hft_retract(): 
 *	accept "release the screen" signal, 
 *	save the display,
 *	wait for permission to use screen again,
 * restore the display
 */
private int
hft_retract()
{
    (void) signal(SIGRETRACT, hft_retract);
 
#ifdef apa16
    apa16_SaveState();
#endif
    if (write(fd_hft, &hft_scn_release, sizeof(hft_scn_release)) != 
	sizeof(hft_scn_release)) {
	(void) fprintf(stderr, "hft_request: Write failed on `/dev/hft' (%s)\n",
	    PERROR);
	return(-1);
    }

    hft_request(fd_hft);
#ifdef apa16
    apa16_RestoreState();
#endif
} /* end hft_retract */
   
/* hft_open():
 *	Open the hft, put it in monitor mode, and grant permission to
 *	the bus
 */
public int
hft_open()
{
    struct hwdbase base;
    int hft;
 
    /*
     * Open the bus
     */
    if ((fd_bus = open("/dev/bus", O_RDWR)) == -1) {
	(void) fprintf(stderr, "hft_open: Cannot open `/dev/bus' (%s)\n", 
	    PERROR);
	return(-1);
    }

    if (ioctl(fd_bus, HWDBASE, &base) == -1) {
	(void) fprintf(stderr, "hft_open: Ioctl failed on `/dev/bus' (%s)\n", 
	    PERROR);
	(void) close(fd_bus);
	return(-1);
    }

    /*
     * Open the hft
     */
    if ((hft = open("/dev/hft", O_RDWR)) == -1) {
	(void) fprintf(stderr, "hft_open: Cannot open `/dev/hft' (%s)\n", 
	    PERROR);
	(void) close(fd_bus);
	return(-1);
    }

    /* 
     * establish screen "grant" and "retract" signal catchers 
     */
    (void) signal(SIGGRANT, hft_grant);
    (void) signal(SIGRETRACT, hft_retract);
 
    /* 
     * indicate that this process is to receive monitor signals 
     */
    if (ioctl(hft, HFSMON, &hft_set_signals) == -1) {
	(void) fprintf(stderr, "hft_open: Ioctl failed on `/dev/hft' (%s)\n",
	    PERROR);
	(void) close(fd_bus);
	(void) close(hft);
	return(-1);
    }

    /* 
     * enter monitor mode 
     */
    if (write(hft, &hft_monitor_mode, sizeof(hft_monitor_mode)) != 
	sizeof(hft_monitor_mode)) {
	(void) fprintf(stderr, "hft_open: Write failed on `/dev/hft' (%s)\n",
	    PERROR);
	(void) close(fd_bus);
	(void) close(hft);
	return(-1);
    }

    /* 
     * indicate how we want the keyboard to be handled 
     */
    if (ioctl(hft, HFSECHO, &hft_set_keys) == -1) {
	(void) fprintf(stderr, "hft_open: Ioctl failed on `/dev/hft' (%s)\n",
	    PERROR);
	(void) close(fd_bus);
	(void) close(hft);
	return(-1);
    }
   
    /* 
     * request screen 
     */
    if (hft_request(hft) == -1) {
	(void) close(fd_bus);
	(void) close(hft);
	return(-1);
    }
 
    fd_hft = hft;
    return(hft);
} /* end hft_open */
 
/* hft_close():
 *	close the graphics window and release the screen
 */
public void
hft_close(hft)
{
    /* 
     * release the screen 
     */
    if (write(hft, &hft_scn_release, sizeof(hft_scn_release)) != 
	sizeof(hft_scn_release)) {
	(void) fprintf(stderr, "hft_close: Write failed on `/dev/hft' (%s)\n",
	    PERROR);
	(void) close(hft);
	(void) close(fd_bus);
	return;
    }
   
    /* 
     * indicate this process is no longer to receive monitor signals 
     */
    if (ioctl(hft, HFCMON, 0) == -1) {
	(void) fprintf(stderr, "hft_open: Ioctl failed on `/dev/hft' (%s)\n",
	    PERROR);
	(void) close(fd_bus);
	(void) close(hft);
	return;
    }
    (void) close(hft);
} /* end hft_close */
 
/* hft_mouse():
 *	test for mouse activity
 */
public struct hflocator *
hft_mouse(hft)
{
    private struct hflocator hft_locator =		/* "read locator" */
    { HFINTROESC, HFINTROLBR, HFINTROWHY, };

    if (read(fd_hft, &hft_locator, sizeof(hft_locator)) > 0) {
#ifdef DEBUG
        int  lt, rt;
        short x, y;

        lt = (hft_locator.hf_buttons & HFBUTTON1) != 0;
        rt = (hft_locator.hf_buttons & HFBUTTON2) != 0;
        x  = (hft_locator.hf_deltax[0] << 8) + hft_locator.hf_deltax[1];
        y  = (hft_locator.hf_deltay[0] << 8) + hft_locator.hf_deltay[1];
        (void) fprintf(stderr, "hft_mouse: left=%d right=%d x=%d y=%d\n", 
  	    lt, rt, x, y);
#endif 
       return(&hft_locator);
   }
   else
	return(NIL(struct hflocator));
} /* end hft_mouse */
 
/* hft_wait():
 * 	set Hft device for read-with/without-blocking
 */
private
hft_wait(flag)
int flag;
{
    if (flag && !hft_waitflag)
        /* 
         * force read's to block until keyboard or locator response 
         */
        if (fcntl(fd_hft, F_SETFL, fcntl(fd_hft, F_GETFL) & ~O_NDELAY) != 0) {
	    (void) fprintf(stderr, 
		"hft_wait: Fcntl failed on `/dev/hft' (%s)\n", PERROR);
	    return;
	}
    if (!flag && hft_waitflag)
        /* 
         * force read's to return immediately if no keyboard or locator response
         */
        if (fcntl(fd_hft, F_SETFL, fcntl(fd_hft, F_GETFL) | O_NDELAY) != 0) {
	    (void) fprintf(stderr, 
		"hft_wait: Fcntl failed on `/dev/hft' (%s)\n", PERROR);
	    return;
	}
    hft_waitflag = flag;
} /* hft_wait */ 


/* hft_key():
 *	Test for keyboard activity
 *    	Returned:	scan code (0 indicates no key hit)
 */
public int
hft_key()
{
    private struct hfunxlate hft_keyboard =		/* "read keyboard" */
        { HFINTROESC, HFINTROLBR, HFINTROWW };
    private int make;
   
    forever() { 
       if (read(fd_hft, &hft_keyboard, sizeof hft_keyboard) <= 0) { 
           /* nothing read */
	   if (hft_waitflag) 
	       continue;
           return(0);
       }
 
       return(hft_keyboard.hf_scancode);
    }
} /* end hft_key */
 
