#include "line.h"
#include "shape.h"
#include "rand.h"
#include "asteroids.h"
#include "ship.h"
#include "param.h"
#include "soundeff.h"
#include "alien.h"

#if defined(SUN) || defined(HP)
#define VELUNIT 113
#endif

#ifndef VELMULT
#define VELMULT 250
#endif

#ifndef VELUNIT
#define VELUNIT 100
#endif

extern long isqrt();


#if defined(SUN) || defined(HP)
/*
 * Larger screens
 */
static LINE AlienShape[] = {
    {{-11, 11}, {11, 11}},
    {{11, 11}, {23, 0}},
    {{-23, 0}, {-11, 11}},
    {{-23, 0}, {23, 0}},
    {{-23, 0}, {-11, -8}},
    {{23, 0}, {11, -8}},
    {{-11, -8}, {11, -8}},
    {{-8, -8}, {-5, -15}},
    {{-5, -15}, {5, -15}},
    {{5, -15}, {8, -8}}
};
/*
 * Smaller screens
 */
/*#if defined(APA16) || defined(MIPSCOLOR) || defined(MPEL) || defined(aiws)*/
#else
static LINE AlienShape[] = {
    {{-10, 10}, {10, 10}},
    {{10, 10}, {20, 0}},
    {{-20, 0}, {-10, 10}},
    {{-20, 0}, {20, 0}},
    {{-20, 0}, {-10, -7}},
    {{20, 0}, {10, -7}},
    {{-10, -7}, {10, -7}},
    {{-7, -7}, {-4, -13}},
    {{-4, -13}, {4, -13}},
    {{4, -13}, {7, -7}}
};
#endif

static SHAPE *AlienShapes[ALIENSIZES];

void alien_GlobalInit()
{
    LINE *l;
    int i, count;
    FRAC f;

    AlienShapes[0] = shape_Create();
    count = sizeof(AlienShape) / sizeof(LINE);
    for (l = AlienShape; count-- > 0; l++)
        shape_Insert(AlienShapes[0], l);
            
    frac_SetD(&f, ALIENSIZES);
    for (i = 1; i < ALIENSIZES; i++) {
        frac_SetN(&f, ALIENSIZES - i);
        AlienShapes[i] = shape_Create();
        shape_Copy(AlienShapes[i], AlienShapes[0]);
        shape_Scale(AlienShapes[i], &f);
    }
}

ALIEN *alien_Create()
{
    ALIEN *a = New(ALIEN);
    VECT screen;

    plot_GetSize(&screen);

    a->sprite = sprite_Create();
    a->missiles = mlist_Create();
    a->debris = debris_Create(0,10,40);

    sprite_SetShapes(a->sprite, AlienShapes, ALIENSIZES);
    sprite_SetBounds(a->sprite,
      1, vect_GetX(&screen) - 2, SCOREBARH + 1, vect_GetY(&screen) - 2);
    sprite_SetWrap(a->sprite, FALSE);
    sprite_SetBounce(a->sprite, FALSE);

    a->state = alien_GONE;

    return a;
}

void alien_Destroy(a)
ALIEN *a;
{
    sprite_Destroy(a->sprite);
    mlist_Destroy(a->missiles);
    debris_Destroy(a->debris);
    Zap(a);
}

void alien_Start(a)
ALIEN *a;
{
    VECT pos, vel;
    long xl, xh, yl, yh;
    int size;
    int fromside;

    size = rand_Natural(ALIENSIZES);
    fromside = shape_GetRadius(AlienShapes[size]) + 5;
    sprite_GetBounds(a->sprite, &xl, &xh, &yl, &yh);

    a->state = alien_ALIVE;

    /* Determine whether to come on left or right side of screen */

    if (rand_Natural(2) == 0) {
        vect_SetX(&pos, xl + fromside);
        vect_SetX(&vel, 2+size);
    } else {
        vect_SetX(&pos, xh - fromside);
        vect_SetX(&vel, -2-size);
    }

    vect_SetY(&pos, rand_Natural(yh - 2*fromside) + fromside);

    /* If near top of screen, start moving down, else up */

    if (vect_GetY(&pos) < (yl + yh) / 2)
        vect_SetY(&vel, rand_Natural(3));
    else
        vect_SetY(&vel, -rand_Natural(3));

    sprite_SetPos(a->sprite, &pos);
    sprite_SetVel(a->sprite, &vel);
    sprite_SetIndex(a->sprite, size);
    sprite_On(a->sprite);

    mlist_On(a->missiles);
}

void alien_On(a)
     ALIEN *a;
{
    if (a->state == alien_ALIVE) {
	sprite_On(a->sprite);
	sprite_Update(a->sprite);
    }
    else if (a->state == alien_DEBRIS) {
	debris_On(a->debris);
	debris_Update(a->debris);
    }
    if (mlist_GetCount(a->missiles) > 0) {
	mlist_On(a->missiles);
	mlist_Update(a->missiles);
    }
}

void alien_Off(a)
     ALIEN *a;
{
    if (a->state == alien_ALIVE) {
	sprite_Off(a->sprite);
	sprite_Update(a->sprite);
    }
    else if (a->state == alien_DEBRIS) {
	debris_Off(a->debris);
	debris_Update(a->debris);
    }
    if (mlist_GetCount(a->missiles) > 0) {
	mlist_Off(a->missiles);
	mlist_Update(a->missiles);
    }
}    
    
void alien_Update(a, s, lvl)
     ALIEN *a;
     SHIP *s;
     int lvl;
{
    VECT vel, dir, pos, svel;
    int dsc;
    FRAC f;

    switch (a->state) {
      case alien_ALIVE:
        /* The sprite is non-wrapping and turns off by itself */

        /* If it goes out of the asteroid field boundaries. */

        if (! sprite_IsOn(a->sprite)) {
	    /* The state is changed DEBRIS instead of GONE so that shots keep
	     * getting tracked.
	     */
            a->state = alien_DEBRIS;
            break;
        }

        /*
	 * Fire towards the player
	 */
        if (mlist_GetCount(a->missiles) == 0) {

            /* Fire a missile */
	    /*
	     * Aim at the ship
	     */
	    ship_GetPos(s, &dir);
            sprite_GetPos(a->sprite, &pos);
	    vect_Sub(&dir, &pos);
	    dsc = vect_SumSqr(&dir); 
	    /*
	     * Add a random probability of missing
	     * which is lower at higher levels
	     */
	    switch(rand_Natural(10)) {
	    case 0:
	    case 1:
	    case 2:
		/*
		 * Deviate at most by five degrees
		 */
		cordic_Rotate(&dir, 5 - lvl > 0 ? rand_Natural(5 - lvl) : 0);
		break;
	    case 3:
	    case 4:
	    case 5:
		/*
		 * Deviate at most by 22 degrees
		 */
		cordic_Rotate(&dir, 22 - lvl * 2 > 0 ? 
				rand_Natural(22 - lvl * 2) : 0);
		break;
	    case 6:
	    case 7:
		/*
		 * Deviate at most by 45 degrees
		 */
		cordic_Rotate(&dir, 45 - lvl * 4 > 0 ? 
			    rand_Natural(45 - lvl * 4) : 0);
		break;
	    case 8:
		/*
		 * Deviate at most by 90 degrees
		 */
		cordic_Rotate(&dir, 90 - lvl * 9 > 0 ? 
			    rand_Natural(45 - lvl * 9) : 0);
		break;
	    case 9:
		/*
		 * Random shot
		 */
		cordic_Rotate(&dir, rand_Natural(360));
		break;
	    default:
		break;
	    }
	    
	    

            vect_MulConst(&dir, ALIENSHOTSPEED * VELUNIT);
	    vect_DivConst(&dir, isqrt(dsc));
            sprite_GetVel(a->sprite, &vel);
            vect_Add(&vel, &dir);
	    ship_GetVel(s, &svel);
	    vect_Add(&vel, &svel);
            frac_SetND(&f, 1, VELMULT);
            mlist_Fire(a->missiles, &pos, &vel, &f, ALIENSHOTRANGE, 
		shape_GetRadius(sprite_GetCurShape(a->sprite)));
        }

        if (rand_Natural(ALIENCLUMSINESS) == 0) {
            sprite_GetVel(a->sprite, &vel);
            vect_SetY(&vel, 2*rand_Integer(2));
            sprite_SetVel(a->sprite, &vel);
        }
        break;

      case alien_DEBRIS:
        debris_Update(a->debris);
        if (mlist_GetCount(a->missiles) == 0 && debris_IsClear(a->debris))
            a->state = alien_GONE;
        break;

      case alien_GONE:
        break;
    }

    if (a->state != alien_GONE)
        sprite_Update(a->sprite);

    if (mlist_GetCount(a->missiles) > 0)
        mlist_Update(a->missiles);
}

/*
 * Cancel also cancels alien shots and debris
 */

void alien_Cancel(a)
ALIEN *a;
{
    sprite_Off(a->sprite);
    mlist_Off(a->missiles);
    debris_Off(a->debris);
    a->state = alien_GONE;
}

static void die(a)
ALIEN *a;
{
    VECT v;

    sprite_Off(a->sprite);
    a->state = alien_DEBRIS;
    sprite_GetPos(a->sprite, &v);
    debris_SetPos(a->debris, &v);
    sprite_GetVel(a->sprite, &v);
    debris_SetVel(a->debris, &v);
    debris_Start(a->debris);
}

/*
 * HitMissile returns points scored and kills alien, if alien is hit.
 */

int alien_HitMissile(a, m)
ALIEN *a;
MISSILE *m;
{
    if (missile_HitSprite(m, a->sprite)) {
        die(a);
        soundeff_AlienPoints();
        return BASEALIENSCORE + MULALIENSCORE * sprite_GetIndex(a->sprite);
    }
    return 0;
}

struct rock {
    ASTEROIDS *aster;
    SHIP *ship;
    boolean died;
};

static void checkMissile(m, rock)
MISSILE *m;
struct rock *rock;
{
    if (asteroids_HitMissile(rock->aster, m) != 0) {
        missile_Off(m);
        return;
    }

    if (rock->ship != NULL &&
      missile_HitSprite(m, ship_GetSprite(rock->ship)) &&
      ! rock->died && ! ship_IsInHyperspace(rock->ship)) {
        missile_Off(m);
        rock->died = TRUE;
    }
}

boolean alien_CheckCollisions(alien, aster, ship, score)
ALIEN *alien;
ASTEROIDS *aster;
SHIP *ship;
int *score;
{
    VECT pos, shippos;
    struct rock rock;

    rock.aster = aster;
    rock.ship = ship;
    rock.died = FALSE;

    if (alien->state == alien_ALIVE &&
      ship != NULL && ! ship_IsInHyperspace(ship)) {
        sprite_GetPos(alien->sprite, &pos);
        ship_GetPos(ship, &shippos);
        if (shape_Collision(sprite_GetCurShape(alien->sprite),
          &pos, ship_GetCurShape(ship), &shippos)) {
            rock.died = TRUE;
            die(alien);
            soundeff_AlienPoints();
            *score = BASEALIENSCORE +
              MULALIENSCORE * sprite_GetIndex(alien->sprite);
        }
    }

    if (alien->state == alien_ALIVE &&
      asteroids_HitSprite(aster, alien->sprite) != 0) {
        die(alien);
        soundeff_AlienExplode();
    }

    mlist_Enumerate(alien->missiles, checkMissile, &rock);
    return rock.died;
}
