#include "type.h"

extern u_short *_apa16_Queue;
extern int _apa16_oldX, _apa16_oldY;
extern u_short *_apa16_SaveArea;

/* Memory addresses on apa16: */

#define apa16_Base (char *)0xf4d80000
#define apa16_QueueMax (u_short *)0xf4d9f6fe
#define apa16_QueueMin (u_short *)0xf4d9c000
#define apa16_CursorX (*(u_short *)0xf4d9f800)
#define apa16_CursorY (*(u_short *)0xf4d9f802)
#define apa16_QueueCounter (*(u_short *)0xf4d9f804)
#define apa16_QueuePtr (*(u_short *)0xf4d9f806)

#define apa16_ScanLine (u_short *)0xf4d9f808
#define apa16_ModeShadow (u_short *)0xf4d9f812

#define apa16_AddrToQueue(addr) (((u_long)addr >> 1) & 0xffff)
#define apa16_QueueToAddr(queue) (u_short *)((queue << 1) | (u_long)apa16_Base)

/* IO addresses. */

#define apa16_EnableOutput() (*(short *)0xf0000d1a = 0)
#define apa16_DisableOutput() (*(short *)0xf0000d1a)
#define apa16_Control(value) (*(short *)0xf0000d12 = (short)value)
#define apa16_Status() (*(short *)0xf0000d12)
#define apa16_IncQueueCounter() (*(short *)0xf0000d14 = 0)
#define apa16_ResetAdapter() (*(short *)0xf0000d20 = 0)

#define apa16_Queue(op) \
    *_apa16_Queue-- = op;

#define apa16_DecQueueCounter 0x800
#define apa16_Load(reg, value) apa16_Queue((((reg)&0xf)<<12)|((value)&0x3ff))
#define apa16_Execute(op, fn) apa16_Queue(0xd000|(((op)&0x3f)<<4)|((fn)&0xf))
#define apa16_Branch(addr) apa16_Queue(0xe000 | (((u_long)addr >> 1)&0x1fff))

#define apa16_Call() apa16_Queue(0xd3b0)
#define apa16_Return() apa16_Queue(0x3c00)

#define op_RelDrawNull 0x2c
#define op_RelDraw 0x2d
#define op_RelMove 0x2e
#define op_Fill 0x2f
#define op_Copy 0x30
#define op_Draw 0x35
#define op_DrawNull 0x36

#ifndef NULL
#define NULL 0
#endif NULL

int apa16_Init();

void apa16_Fill();
void apa16_Copy();
void apa16_DrawNull();
void apa16_RelMove();
void apa16_RelDraw();
void apa16_RelDrawNull();
void apa16_Run();

#define fn_Zero 0x0
#define fn_And 0x1
#define fn_Copy 0x9
#define fn_Xor 0xa
#define fn_Or 0xb
#define fn_One 0xf

#ifdef notdef
#define apa16_Check(space) \
{ \
    short q; \
    if (_apa16_Queue <= apa16_QueueMin + space) { \
        q = apa16_AddrToQueue(_apa16_Queue); \
        while (q == apa16_QueuePtr) \
            ; \
        *_apa16_Queue = apa16_Branch(apa16_QueueMax); \
        _apa16_Queue = apa16_QueueMax; \
    } \
    q = apa16_AddrToQueue(_apa16_Queue); \
    if (apa16_QueueCounter != 0 && q >= apa16_QueuePtr) { \
        q -= space; \
        while (q >= apa16_QueuePtr) \
            ; \
    } \
}
#else
#define apa16_Check(a) { int i; for(i = 0; i < 1000; i++);}
#endif

#define apa16_Move(x, y) { \
    apa16_Check(2); \
    apa16_Load(0x1, x); \
    apa16_Load(0x0, y); \
    _apa16_oldX = x; \
    _apa16_oldY = y; \
}
    
#define apa16_Draw(x, y, fn) { \
    if (x == _apa16_oldX) { \
        if (y < _apa16_oldY) \
            apa16_Fill(x, y, 1, _apa16_oldY - y + 1, fn); \
        else \
            apa16_Fill(x, _apa16_oldY, 1, y - _apa16_oldY + 1, fn); \
        _apa16_oldX = _apa16_oldY = -1; \
    } else { \
        apa16_Check(3); \
        apa16_Load(0x5, x); \
        apa16_Load(0x6, y); \
        apa16_Execute(op_Draw, fn); \
        _apa16_oldX = x; \
        _apa16_oldY = y; \
    } \
}

#define apa16_SaveState() { \
    while (apa16_QueueCounter > 0) \
        ; \
    bcopy(apa16_Base, _apa16_SaveArea, 1024 * 768 / 8); \
}

#define apa16_RestoreState() { \
    apa16_ResetAdapter();	\
    apa16_Control(apa16_Status() | 0x400);	\
    apa16_EnableOutput();	\
    _apa16_Queue = apa16_QueueMax;	\
    apa16_QueuePtr = apa16_AddrToQueue(_apa16_Queue);	\
    bcopy(_apa16_SaveArea, apa16_Base, 1024 * 768 / 8); \
    apa16_Move(_apa16_oldX, _apa16_oldY); \
}
