/*
 * Generic bitmapter drawing
 */

#include "bitmap.h"

#ifndef BM_WORD
#define BM_WORD u_long
#endif

#define BM_WORD_WIDTH (sizeof(BM_WORD)*8)
#define BM_WORD_MASK  ((BM_WORD) BM_WORD_WIDTH - 1)

#define MSBit (1<<(BM_WORD_WIDTH-1))
#define LSBit 1

#ifndef BM_BLACK_ON_WHITE

/* a 1 bit is black */
#define MERGE_SET(w,m) (*(w)&=~(m))
#define MERGE_CLR(w,m) (*(w)|=(m))
#define MERGE_INV(w,m) (*(w)^=(m))
#define BM_WHITE_WORD  ((BM_WORD)0)
#define BM_BLACK_WORD  ((BM_WORD)~0)

#else

/* a 1 bit is white */
#define MERGE_SET(w,m) (*(w)|=(m))
#define MERGE_CLR(w,m) (*(w)&=~(m))
#define MERGE_INV(w,m) (*(w)^=(m))
#define BM_WHITE_WORD  ((BM_WORD)~0)
#define BM_BLACK_WORD  ((BM_WORD)0)

#endif

/*
 * Creating, destroying, copying bitmaps
 */

struct bitmap *bitmap_Create(w, h)
int w, h;
{
    struct bitmap *bitmap = (struct bitmap *)
      malloc(sizeof (struct bitmap));

    bitmap->width = w;
    bitmap->height = h;
    bitmap->freeBits=TRUE;

    bitmap->bits = (unsigned char *)
	malloc((w + (BM_WORD_WIDTH - 1)) / BM_WORD_WIDTH * h * sizeof (BM_WORD));

    return bitmap;
}

/*
 * Make raster from display device
 */

struct bitmap *bitmap_Display(base, w, h)
unsigned char *base;
int w, h;
{
    struct bitmap *bitmap = (struct bitmap *)
      malloc(sizeof (struct bitmap));

    bitmap->width = w;
    bitmap->height = h;
    bitmap->bits = base;
    bitmap->freeBits=FALSE;

    return bitmap;
}

void bitmap_Destroy(bitmap)
struct bitmap *bitmap;
{
    if(bitmap->freeBits)
	free(bitmap->bits);
    free(bitmap);
}

void bitmap_ClearWhite(bitmap)
struct bitmap *bitmap;
{
    register BM_WORD b, *p;
    register int count;

    p = (BM_WORD *)bitmap->bits;
    count =
	(bitmap->width + (BM_WORD_WIDTH - 1)) / BM_WORD_WIDTH * bitmap->height;
    b = BM_WHITE_WORD;

    while (count--)
        *p++ = b;
}

void bitmap_ClearBlack(bitmap)
struct bitmap *bitmap;
{
    register BM_WORD b, *p;
    register int count;

    p = (BM_WORD *)bitmap->bits;
    count =
	(bitmap->width + (BM_WORD_WIDTH - 1)) / BM_WORD_WIDTH * bitmap->height;
    b = BM_BLACK_WORD;

    while (count--)
        *p++ = b;
}

void bitmap_Invert(bitmap)
struct bitmap *bitmap;
{
    register BM_WORD b, *p;
    register int count;

    p = (BM_WORD *)bitmap->bits;
    count =
	(bitmap->width + (BM_WORD_WIDTH - 1)) / BM_WORD_WIDTH * bitmap->height;
    b = BM_WORD_MASK;

    while (count--)
        *p++ ^= b;
}

/*
 * Drawing Lines
 *
 * The algorithm has been divided into 24 cases for speed
 * (Actually only 6 are necessary.  I screwed up.)
 * Point addresses and bits are computed incrementally.
 */

#ifdef BM_MSBLEFT
#define LEFT(s,b)	(b == MSBit ? s--, (b = LSBit) : (b <<= 1))
#define RIGHT(s,b)	(b == LSBit ? s++, (b = MSBit) : (b >>= 1))
#else
#define LEFT(s,b)	(b == LSBit ? s--, (b = MSBit) : (b >>= 1))
#define RIGHT(s,b)	(b == MSBit ? s++, (b = LSBit) : (b <<= 1))
#endif

#define UP(s,b)		(s -= linelen)
#define DOWN(s,b)	(s += linelen)

#define MERGE	MERGE_SET
void bitmap_SetLine(r, x0, y0, x1, y1)
#include "bitmap_line.h"
#undef MERGE

#define MERGE	MERGE_CLR
void bitmap_ResLine(r, x0, y0, x1, y1)
#include "bitmap_line.h"
#undef MERGE

#define MERGE	MERGE_INV
void bitmap_InvLine(r, x0, y0, x1, y1)
#include "bitmap_line.h"
#undef MERGE

#undef LEFT
#undef RIGHT
#undef UP
#undef DOWN
