/*
 * Generic byte wide color drawing
 */

#include "bytemap.h"

#ifdef BM_BLACK_ON_WHITE
# define BM_BLACK_WORD	0x00000000
# define BM_WHITE_WORD	0x01010101
# define _BLACK		1
# define _WHITE		0
#else
# define BM_BLACK_WORD	0x01010101
# define BM_WHITE_WORD	0x00000000
# define _BLACK		0
# define _WHITE		1
#endif
/*
 * Creating, destroying, copying rasters
 */

struct bytemap *bytemap_Create(w, h)
int w, h;
{
    struct bytemap *bytemap = (struct bytemap *)
      malloc(sizeof (struct bytemap));

    bytemap->width = w;
    bytemap->height = h;

    bytemap->bytes = (u_char *)
      malloc(w * h);
    bytemap->freeBytes = TRUE;

    return bytemap;
}

/*
 * Make bytemapter from display device
 */

struct bytemap *bytemap_Display(base, w, h)
u_char *base;
int w, h;
{
    struct bytemap *bytemap = (struct bytemap *)
      malloc(sizeof (struct bytemap));

    bytemap->width = w;
    bytemap->height = h;
    bytemap->bytes = base;
    bytemap->freeBytes = FALSE;

    return bytemap;
}

/*
 * Do not use on Display device rasters!
 * This is not true now that I added the freeBytes flag.
 */

void bytemap_Destroy(bytemap)
struct bytemap *bytemap;
{
    if (bytemap->freeBytes)
        free(bytemap->bytes);
    free(bytemap);
}

void bytemap_ClearWhite(bytemap)
struct bytemap *bytemap;
{
    register u_long b, *p;
    register int count;

    p = (u_long *)bytemap->bytes;
    count = (bytemap->width * bytemap->height) / sizeof(long);
    b = BM_WHITE_WORD;
    while (count--)
        *p++ = b;
}

void bytemap_ClearBlack(bytemap)
struct bytemap *bytemap;
{
    register u_long b, *p;
    register int count;

    p = (u_long *)bytemap->bytes;
    count = (bytemap->width * bytemap->height) / sizeof(long);
    b = BM_BLACK_WORD;

    while (count--)
        *p++ = b;
}

void bytemap_Invert(bytemap)
struct bytemap *bytemap;
{
    register u_long b, *p;
    register int count;

    p = (u_long *)bytemap->bytes;
    count = (bytemap->width * bytemap->height) / sizeof(long);
    b = 0x01010101;

    while (count--)
        *p++ ^= b;
}

/*
 * Drawing Lines
 *
 * The algorithm has been divided into 24 cases for speed
 * (Actually only 6 are necessary.  I screwed up.)
 * Point addresses and bits are computed incrementally.
 */
#define LEFT	(s--)
#define RIGHT	(s++)

#define UP	(s -= linelen)
#define DOWN	(s += linelen)

#define LOOP(maxdelta, mindelta, majorop, minorop) {	\
	errterm = 2 * mindelta - maxdelta;		\
	errdec = errterm - maxdelta;		\
	errinc = errterm + maxdelta;			\
	while (maxdelta--) {			\
                if (errterm > 0) {                      \
                        errterm += errdec;              \
			minorop;			\
		} else				\
			errterm += errinc;		\
                majorop;				\
		MERGE;				\
	}					\
}


#define MERGE	(*s = _WHITE)
void bytemap_SetLine(r, x0, y0, x1, y1)
#include "bytemap_line.h"
#undef MERGE

#define MERGE	(*s = _BLACK)
void bytemap_ResLine(r, x0, y0, x1, y1)
#include "bytemap_line.h"
#undef MERGE

#define MERGE	(*s ^= 1)
void bytemap_InvLine(r, x0, y0, x1, y1)
#include "bytemap_line.h"
#undef MERGE

#undef LEFT
#undef RIGHT
#undef UP
#undef DOWN
