#include "type.h"
#include "control.h"
#include "input.h"
#include "param.h"
#include "soundeff.h"

static struct timeval LoopEnds;

void control_BeginLoop()
{
    static int deet = 0;   /* Toggles between 0 and 1 */
    static int deetCount = 0;

    gettimeofday(&LoopEnds, NULL);
    LoopEnds.tv_usec += LOOPPERIOD;
    while (LoopEnds.tv_usec >= 1000000) {
	LoopEnds.tv_sec++;
	LoopEnds.tv_usec -= 1000000;
    }
    if (deetCount++ == DEETPERIOD) {
        deetCount = 0;
        if (deet = 1 - deet)
            soundeff_LowDeet()
        else
            soundeff_HighDeet()
    }
}

enum control_Command control_NextCommand()
{
    enum input_Status is;
    enum input_Key ch;
    struct timeval now, delay;

    delay.tv_sec = 0;

    do {
        gettimeofday(&now, NULL);
        if (timercmp(&LoopEnds, &now, <)) 
	    delay.tv_sec = delay.tv_usec = 0;
	else {
	    delay.tv_usec = (LoopEnds.tv_usec - now.tv_usec);
	    delay.tv_sec = (LoopEnds.tv_sec - now.tv_sec);
	    while (delay.tv_usec < 0) {
		delay.tv_sec--;
		delay.tv_usec += 1000000;
	    }
	}

        is = input_Poll(&ch,&delay);

        switch (is) {
            case input_KEYUP:
                switch (ch) {
                    case key_QUIT:
                        return control_QUIT;

                    case key_THRUST:
                        return control_THRUSTOFF;

                    case key_ROTLEFT:
                        return control_ROTATELEFTOFF;

                    case key_ROTRIGHT:
                        return control_ROTATERIGHTOFF;

		    case key_START:
			return control_START;

		    case key_START_2:
			return control_START_2;

                    default:
                        break;
                }
                break;

            case input_KEYDOWN:
                switch (ch) {
                    case key_ROTLEFT:
                        return control_ROTATELEFTON;

                    case key_FIRE:
                        return control_FIRE;

		    case key_PAUSE:
			return control_PAUSE;

                    case key_TOGGLESOUND:
                        return control_TOGGLESOUND;

                    case key_ROTRIGHT:
                        return control_ROTATERIGHTON;

                    case key_THRUST:
                        return control_THRUSTON;

                    case key_HYPERSPACE:
                        return control_HYPERSPACE;

                    default:
                        break;
                }
                break;

            default:
                break;
	    }
    } while (delay.tv_usec != 0);

    return control_TIMEUP;
}

