#include "cordic.h"
#include "type.h"
#include "vect.h"

/*
 * Cordic implementation
 * Modified from a version by:
 * paul@torch.UUCP (Paul Andrews)
 */

private long arctan[] = {    /* atans in deg << 16 */
    2949120, 1740967, 919879, 466945,
    234378, 117303, 58666, 29335,
    14668, 7334, 3667, 1833,
    918, 458, 229, 115
};

/*
 * Theta: between -90 and 90 (deg), left-shifted by 16 bits
 *
 * Performs:
 *     Vx = Vx * cos(theta) - Vy * sin(theta)
 *     Vy = Vy * cos(theta) + Vx * sin(theta)
 */

private void small_rot(v, theta)
VECT *v;
long theta;
{
    register long x, y, oldX, i;

    x = vect_GetX(v) << 8;
    y = vect_GetY(v) << 8;

    for (i = 0; i < 16; i++) {
        oldX = x >> i;
        if (theta < 0) {
            x += y >> i;
            y -= oldX;
            theta += arctan[i];
        } else {
            x -= y >> i;
            y += oldX;
            theta -= arctan[i];
        }
    }

    vect_SetXY(v, (x << 8) / KR, (y << 8) / KR);
}


void cordic_Rotate(v, theta)
VECT *v;
int theta;
{
    int x, y;

    while (theta < -90)
        theta += 360;
    while (theta > 270)
        theta -= 360;

    if (theta > 90) {
        vect_SetXY(v, -vect_GetX(v), -vect_GetY(v));
        theta -= 180;
    }

    if (theta == 90) {
        x = vect_GetX(v);
        y = vect_GetY(v);
        vect_SetXY(v, -y, x);
    }
    else if (theta == -90) {
        x = vect_GetX(v);
        y = vect_GetY(v);
        vect_SetXY(v, y, -x);
    }
    else if (theta != 0)
        small_rot(v, theta << 16);
}
