#include "type.h"
#include "debris.h"
#include "rand.h"
#include "shape.h"
#include "line.h"
#include "sprite.h"
#include "param.h"

#define RADIUSNUM 10
#define RADIUSDEN 5

static SHAPE *shards[SHARDMAX];

void debris_GlobalInit()
{
    static LINE line={{-SHARDMAXLEN*2/3,0},{SHARDMAXLEN/3,0}};
    static VECT scale={SHARDMAX,SHARDMAX};
    int i;

    shards[0]=shape_Create();
    shape_Insert(shards[0],&line);

    for(i=1; i<SHARDMAX; i++){
	shards[i]=shape_Create();
	shape_Copy(shards[i],shards[0]);
	shape_Rotate(shards[i],i*SHARDSPIN);
	shape_Scale(shards[i],&scale);
	vect_SetY(&scale,vect_GetY(&scale)+SHARDMAXLEN);
    }
}

DEBRIS *debris_Create(size,num,duration)
int size,num,duration;
{
    VECT screen;
    DEBRIS *d=New(DEBRIS);
    int i,numshards;;

    plot_GetSize(&screen);

    d->offs=SHARDMAXLEN-size;
    if(d->offs>=SHARDMAXLEN || d->offs<0)
	d->offs=0;
    numshards=SHARDMAX-d->offs;

    d->num=num;
    if(d->num>MAXDEBRIS || d->num<=0)
	d->num=MAXDEBRIS;
    if(d->num>numshards)
	d->num=numshards;

    if(duration>numshards || duration<=0)
	duration=numshards;

    for(i=0; i<d->num; i++){
	d->pieces[i]=sprite_Create();
        sprite_SetShapes(d->pieces[i],
			 shards+d->offs+i/*random?*/,
			 numshards-i);
	sprite_SetInc(d->pieces[i],numshards/duration);
        sprite_SetBounds(d->pieces[i],
          1,vect_GetX(&screen)-2,SCOREBARH+1,vect_GetY(&screen)-2);
        sprite_SetWrap(d->pieces[i],FALSE);
        sprite_SetBounce(d->pieces[i],FALSE);
    }

    vect_SetXY(&d->vel,0,0);
    d->left=0;
    
    return d;
}

void debris_Start(d)
DEBRIS *d;
{
    int i;
    
    frac_SetND(&d->radius,RADIUSNUM,RADIUSDEN);
    
    for(i=0; i<d->num; i++){
	vect_SetXY(&d->directions[i],
		   rand_Integer(SHARDVEL),
		   rand_Integer(SHARDVEL));
	sprite_SetIndex(d->pieces[i],0);
	sprite_On(d->pieces[i]);
	d->fizzled[i]=FALSE;
    }
    
    d->left=d->num;
}

void debris_On(d)
DEBRIS *d;
{
}

void debris_Off(d)
DEBRIS *d;
{
    int i;
    for(i=0; i<d->num; i++)
	sprite_Off(d->pieces[i]);
}

void debris_Destroy(d)
DEBRIS *d;
{
    int i;
    for(i=0; i<d->num; i++)
	sprite_Destroy(d->pieces[i]);
    Zap(d);
}

void debris_Update(d)
DEBRIS *d;
{
    if(d->left>0){
	int i;
	
	for(i=0; i<d->num; i++){
	    SPRITE *s=d->pieces[i];
	    if(!d->fizzled[i])
		if(!sprite_IsOn(s) ||
		   sprite_GetIndex(s)>sprite_GetNumShapes(s)-sprite_GetInc(s)-1){
		    sprite_Off(s);
		    d->fizzled[i]=TRUE;
		    d->left--;
		}else{
		    VECT pos;
		    vect_Copy(&pos,&d->directions[i]);
		    vect_Scale(&pos,&d->radius);
		    vect_Add(&pos,&d->pos);
		    sprite_SetPos(s,&pos);
		}
	    
	    sprite_Update(s);
	}
	
	frac_SetN(&d->radius,frac_GetN(&d->radius)+1);
	vect_Add(&d->pos,&d->vel);
    }
}
