#include <stdio.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/time.h>
#if defined(aiws) || defined(sun) || defined(hpux)
#include <fcntl.h>
#ifdef aiws
#include <sys/lockf.h>
#else
#include <unistd.h>
#endif
#endif /* aiws || sun || hpux */
#include <errno.h>

#include "high.h"

extern int errno;
extern char *index();


#define HIGHCOUNT (HIGHCOLS * HIGHROWS)

#define MAXNAME 10
#define SCOREW 8

#define ESC '\\'
#define SEP ':'
#ifndef LOCK_NB
#define LOCK_NB 0	 /* our lock is non blocking anyway unlike sun's */
#endif


/*
 * Routines dealing with the high score log
 *
 * First, the exclusive access routines
 * For LockLog, mode is same as for open call
 */

#define MAXTRIES 5

static int lockfd;

static boolean LockLog(name, mode)
char *name;
int mode;
{
    int try;

    for (try = 0; try < MAXTRIES; try++) {
        lockfd = open(name, O_RDWR);
        if (lockfd < 0) {
            if (errno == ENOENT)
                close(creat(name, 0666));
#if defined(aiws) || defined(sun) || defined(hpux)
	} else if ( (mode & 3) != O_RDONLY ) {
	    if (lockf(lockfd, F_TLOCK, 0) != -1) {
		close(lockfd);
		return TRUE;
	    }
	    else
		close(lockfd);
	    sleep(1);
	}
	else {
	    close(lockfd);
	    return TRUE;
	}
#else
        } else if ( flock(lockfd,
          ((mode & 3) == O_RDONLY ? LOCK_SH : LOCK_EX) | LOCK_NB) != -1) {
	    close(lockfd);
	    return(TRUE);
	}
	else
	    close(lockfd);
	
#endif
    }

    return FALSE;
}

static void UnlockLog(name)
char *name;
{
    int lockfd;
    lockfd = open(name, O_RDWR);
    if (lockfd == -1)
	return;
#if defined(aiws) || defined(sun) || defined(hpux)
    lockf(lockfd, F_ULOCK, 0);
#else
    flock(lockfd, LOCK_UN);
#endif
    close(lockfd);
}

/*
 * High score log file entry is in the format:
 * 	name:uid:score:time
 * name is any string, uid is the numeric uid, time is a long (unix time)
 *
 * The enumerate proc receives as arguments:
 *   (long rock, static char *name, int uid, long score, long time)
 */

static getNextInt(sep,pP)
int sep;
char **pP;
{
    char *field=index(*pP,sep);

    if(field==NULL)
	return 0;

    *pP=field+1;

    return atoi(*pP);
}

static void DoEnumerate(proc, rock, endrock)
int (*proc)();
long rock, endrock;
{
    char buf[256];
    FILE *fp;
    int i;

    fp = fopen(HIGHSCOREFILE, "r");

    if (!fp) {
	fprintf(stderr, "Can't read score file\n");
	cleanup(-1);
    }

    for ( i = 0; fgets(buf, sizeof (buf), fp) != NULL && i < endrock; i++) {
	char name[MAXNAME+1], *p, *q;
	int uid;
	long score,time;
	int esc=FALSE;

	q=name;
	for(p=buf; *p!='\0' && (*p!=SEP || esc); p++)
	    if(*p==ESC && !esc)
		esc=TRUE;
	    else{
		esc=FALSE;
		if(q-name<MAXNAME)
		    *q++=(*p);
	    }
	*q='\0';

	uid=getNextInt(SEP,&p);
	score=getNextInt(SEP,&p);
	time=getNextInt(SEP,&p);

        (*proc)(rock,name,uid,score,time);
    }

    fclose(fp);
}

static void high_Enumerate(proc, rock)
int (*proc)();
long rock;
{
    if (LockLog(HIGHSCOREFILE, O_RDONLY) == FALSE)
        return;
    DoEnumerate(proc, rock, HIGHCOUNT);
    UnlockLog(HIGHSCOREFILE);
}

static void WriteScore(fp,name,uid,score,time)
FILE *fp;
char *name;
int uid;
long score,time;
{
    char *p;

    for(p=name; *p!='\0' && (p-name)<MAXNAME; p++){
	if(*p==SEP || *p==ESC)
	    putc(ESC,fp);
	putc(*p,fp);
    }

    fprintf(fp,"%c%d%c%d%c%ld\n",SEP,uid,SEP,score,SEP,time);
}

struct scoreEntry {
    char name[MAXNAME];
    int uid;
    int score;
    long time;
};

static int ReadScore(newScoreP,name,uid,score,time)
struct scoreEntry **newScoreP;
char *name;
int uid;
long score,time;
{
    strcpy((*newScoreP)->name, name);
    (*newScoreP)->uid=uid;
    (*newScoreP)->score=score;
    (*newScoreP)->time=time;

    (*newScoreP)++;
}

void high_Record(name,score)
char *name;
int score;
{
    struct scoreEntry scores[HIGHCOUNT], *scoresEnd, *s;
    struct scoreEntry *previous, *earliest;
    int scored;
    long time;
    struct timeval tod;

    int uid;
    
    uid = getuid();
    scoresEnd = &scores[0];
    previous = NULL;
    earliest = NULL;

    gettimeofday(&tod,NULL);
    time=tod.tv_sec;

    if (LockLog(HIGHSCOREFILE, O_RDWR) == FALSE)
        return;

    DoEnumerate(ReadScore,(long) &scoresEnd, (long) HIGHCOUNT);

    for(s=scores; s<scoresEnd; s++)
	if(s->uid==uid){
	    previous=s;
	    break;
	}

    scored=
	(previous!=NULL
	 ? previous->score<score 		/* beat prev score */
	 : ((scoresEnd-scores<HIGHCOUNT) || 	/* room for more */
	    (scoresEnd[-1].score<score)));	/* beat lowest score */

    if(!scored && previous==NULL){
	int timeout=time-SCORETIMEOUT; /* in seconds */

	/* didn't score yet, did we beat a timed-out score? */
	for(s=scores+PERMSCORES; s<scoresEnd; s++)
	    if(s->time<timeout && (earliest==NULL || s->time<=earliest->time))
		earliest=s;

	scored=(earliest!=NULL);
    }

    if(scored){
        FILE *fp = fopen(HIGHSCOREFILE, "w");
	struct scoreEntry *s;
	int written=FALSE;

	if(fp==NULL){
	    fprintf(stderr,"Can't write score file\n");
	    cleanup(-1);
	}

	for(s=scores; s<scoresEnd; s++){
	    if(!written && score > s->score){
		WriteScore(fp,name,uid,score,time);
		written=TRUE;
	    }

	    if(s!=earliest && s!=previous)
		WriteScore(fp,s->name,s->uid,s->score,s->time);
        }

	if(!written)
	    WriteScore(fp,name,uid,score,time);

        fclose(fp);
    }

    UnlockLog(HIGHSCOREFILE);
}


/*
 * Routines dealing with the graphics
 */

static LINEFONT *scoreFont = NULL;
static TEXT *highList[HIGHCOUNT];
static int highCount;
static int cycle;

VECT *scorePos, *scoreSize;
VECT fontCharSpace;
int scoreW, scoreH;
int scoreRow, scoreCol;

static int ProcessScore(rock,name,uid,score,time)
long rock;
char *name;
int uid;
long score,time;
{
    VECT txtpos;
    char scorebuf[SCOREW+1],buf[80];

    if (highCount == HIGHCOUNT)
	return;
    sprintf(scorebuf,"%d",score);
    sprintf(buf,
	    "%*.*s%.*s%s",MAXNAME,MAXNAME,name,
	    (SCOREW-strlen(scorebuf)),".........................",
	    scorebuf);

    highList[highCount] = text_Create();

    text_Set(highList[highCount], buf);
    text_SetFont(highList[highCount], scoreFont);
    vect_SetXY(&txtpos,
      vect_GetX(scorePos) + ((scoreCol+1) * scoreW)
	       - vect_GetX(&fontCharSpace)*MAXNAME,
      vect_GetY(scorePos) + scoreRow * scoreH);
    text_SetPos(highList[highCount], &txtpos);
    text_On(highList[highCount]);

    if (++scoreRow == HIGHROWS) {
        scoreRow = 0;
        scoreCol++;
    }

    highCount++;
}

void high_Display(pos, size)
VECT *pos, *size;
{
    if (scoreFont == NULL) {
        VECT fontSize;
        vect_SetXY(&fontSize, 2, 1);
        scoreFont = linefont_Create(&fontSize, 0);
	linefont_GetCharSpacing(scoreFont,&fontCharSpace);
    }

    scorePos = pos;
    scoreSize = size;
    scoreW = vect_GetX(size) / (HIGHCOLS+1);
    scoreH = vect_GetY(size) / HIGHROWS;
    scoreRow = 0;
    scoreCol = 0;

    highCount = 0;

    high_Enumerate(ProcessScore, 0);

    cycle = 0;
}

void high_Undisplay()
{
    int i;
    for (i = 0; i < highCount; i++)
        text_Destroy(highList[i]);
}

/*
 * Only one is updated per Update
 * on account of the large drawing
 * time required.
 */

void high_Update()
{
    if (highCount == 0) return;
    text_Update(highList[cycle]);
    if (++cycle == highCount)
        cycle = 0;
}
