/*
 * Input driver for IBM RT Advanced Monochrome Display, aix version
 */

#include <stdio.h>

#include "type.h"
#include "input.h"
#include "plot.h"
#include "util.h"

#define GSLBUG
#ifdef GSLBUG
#include <fcntl.h>
#include <sys/hft.h>
#include <termio.h>
#include <sys/time.h>
#include <sys/errno.h>
#include <signal.h>
#include <sys/param.h>
#include <sys/times.h>

#define HFWDCHARS(n) ((n) >>  8) & 0xff,  (n) & 0xff
#define FLWDCHARS(n) ((n) >> 24) & 0xff, ((n) >> 16) & 0xff, \
		     ((n) >>  8) & 0xff,  (n) & 0xff

#define VTD(n) HFINTROESC, HFINTROLBR, HFINTROEX, FLWDCHARS(n)
#define PROTOCOLVTD  VTD(sizeof(struct hfprotocol) - 3)
#define KSRDEFSEL  \
     HFWDCHARS((HFHOSTS<<8)+(HFXLATKBD<<8)+HFWRAP+HFLOCATOR+HFLPFKS+HFDIALS)
#define KSRDEFVAL   HFWDCHARS((HFXLATKBD << 8) + HFWRAP)
#define KSRUNXSEL \
     HFWDCHARS((HFHOSTS << 8) + (HFXLATKBD << 8) + HFLOCATOR + HFLPFKS + HFDIALS)
#define KSRUNXVAL HFWDCHARS(HFHOSTS + HFLOCATOR + HFLPFKS + HFDIALS)
static int ttyfd, nr;
static struct termio tio;
static struct sigvec sv, osv;
static char inbuf[500];
static struct hfprotocol xlat = 
	{ PROTOCOLVTD, HFKSRPROH, HFKSRPROL, 2, 0, KSRDEFSEL, KSRDEFVAL };
static struct hfprotocol unxlat = 
	{ PROTOCOLVTD, HFKSRPROH, HFKSRPROL, 2, 0, KSRUNXSEL, KSRUNXVAL };
static struct hfrconf conf;
int
catch_input(a)
int a;
{
    (*osv.sv_handler)(a);
}
#endif

int hft_fd;
extern char *gsl_error();
extern FILE *plotfd;


boolean input_GlobalInit()
{
#ifndef GSLBUG
    static int six = 6, one = 1;
    int err;

    /*
     * This is a bug in gsl, until we get it fixed, I use my own keyboard
     * raw handler
     */
    if ((err = gseven_(&six)) != 0) {
	(void) fprintf(plotfd, "Could not set up event handling (gseven %s)\n",
	    gsl_error(err));
	return FALSE;
    }
#else

    struct termio ntio;
    /*
     * Open /dev/console to get the keyboard
     */
    if ((ttyfd = open("/dev/console", O_RDWR)) == -1) {
	(void) fprintf(stderr, "Cannot open `/dev/console' (%s)\n", PERROR);
	return FALSE;
    }
    /*
     * Disable `canonical input processing'
     */
    if (ioctl(ttyfd, TCGETA, &tio) == -1) {
	(void) fprintf(stderr, "Ioctl failed on `/dev/console' (%s)\n", PERROR);
	return FALSE;
    }
    ntio = tio;
    ntio.c_lflag &= ~ICANON;
    ntio.c_cc[VMIN] = 7;
    ntio.c_cc[VTIME] = 0;
    if (ioctl(ttyfd, TCSETAF, &ntio) == -1) {
	(void) fprintf(stderr, "Ioctl failed on `/dev/console' (%s)\n", PERROR);
	return FALSE;
    }
#ifndef MPEL
    sigstack(malloc(4096));
    sv.sv_handler = catch_input;
    sv.sv_mask = SIG_STK;
    sigvec(SIGMSG, &sv, &osv);
#else
#ifdef notdef
    /* 
     * force read's to return immediately if no keyboard or locator response
     */
    if (fcntl(ttyfd, F_SETFL, fcntl(ttyfd, F_GETFL) | O_NDELAY) != 0) {
	(void) fprintf(stderr, "Fcntl failed on `/dev/console' (%s)\n", PERROR);
	return FALSE;
    }
#endif
#endif
    /*
     * Setup untranslated keyboard mode
     */
    if (write(ttyfd, &unxlat, sizeof(unxlat)) != sizeof(unxlat)) {
	(void) fprintf(stderr, "Write failed on `/dev/console' (%s)\n", PERROR);
	return FALSE;
    }
    /*
     * Turn click off
     */
    conf.hf_op = HFCHGCLICK;
    conf.hf_obj = 0;
    if ( ioctl(ttyfd, HFRCONF, &conf) == -1 ) {
	(void) fprintf(stderr, 
	    "Ioctl failed on `/dev/console' (HFCHGCLICK) (%s)\n", PERROR);
    }
#ifdef notdef
    /*
     * Set the keyboard rate
     */
    conf.hf_op = HFCHGKBDRATE;
    conf.hf_obj = 39 << 24;
    if ( ioctl(ttyfd, HFRCONF, &conf) == -1 ) {
	(void) fprintf(stderr, 
	    "Ioctl failed on `/dev/console' (HFCHGKBDRATE) (%s)\n", PERROR);
    }
    conf.hf_op = HFCHGKBDDEL;
    conf.hf_obj = 300 << 16;
    if ( ioctl(ttyfd, HFRCONF, &conf) == -1 ) {
	(void) fprintf(stderr, 
	    "Ioctl failed on `/dev/console' (HFCHGKBDDEL) (%s)\n", PERROR);
    }
#endif
#endif
    /*
     * All went well
     */
    return TRUE;
}

/*
 * Note:
 *
 *   Running Asteroids while in ITC's wm is deadly because wm
 *   barfs and locks up the console.
 *   I tried using EISETD with E_STDINPUT in this routine
 *   before exiting, but that turns out to have nothing
 *   to do with the problem.
 */

void input_Done()
{
#ifndef GSLBUG
    static int six = 6;
    int err;
    /* Disable the reporting of keystrokes */
    if ((err = gsevds_(&six)) != 0) {
	(void) fprintf(plotfd, 
	     "Could not disable event reporting (gsevds %s)\n", gsl_error(err));
	return;
    }
#else
    conf.hf_op = HFCHGCLICK;
    conf.hf_obj = 0x80000000;
    if ( ioctl(ttyfd, HFRCONF, &conf) == -1 ) {
	(void) fprintf(stderr, 
	    "Ioctl failed on `/dev/console' (HFCHGCLICK) (%s)\n", PERROR);
    }
#ifdef notdef
    conf.hf_op = HFCHGKBDRATE;
    conf.hf_obj = 40 << 14;
    if ( ioctl(ttyfd, HFRCONF, &conf) == -1 ) {
	(void) fprintf(stderr,
	    "Ioctl failed on `/dev/console' (HFCHGKBDRATE) (%s)\n", PERROR);
    }
    conf.hf_op = HFCHGKBDDEL;
    conf.hf_obj = 400 < 16;
    if ( ioctl(ttyfd, HFRCONF, &conf) == -1 ) {
	(void) fprintf(stderr, 
	    "Ioctl failed on `/dev/console' (HFCHGKBDDEL) (%s)\n", PERROR);
    }
#endif
#ifdef notdef
    conf.hf_op = HFCHGVOLUME;
    conf.hf_obj = 2 << 24;
    if ( ioctl(ttyfd, HFRCONF, &conf) == -1 ) {
	(void) fprintf(stderr, 
	    "Ioctl failed on `/dev/console' (HFCHGVOLUME) (%s)\n", PERROR);
    }
#endif
    /*
     * Setup translated keyboard mode
     */
    if (write(ttyfd, &xlat, sizeof(xlat)) != sizeof(xlat)) {
	(void) fprintf(stderr, "Write failed on `/dev/console' (%s)\n", PERROR);
	return;
    }
#ifdef MPEL
#ifdef notdef
    if (fcntl(ttyfd, F_SETFL, fcntl(ttyfd, F_GETFL) & ~O_NDELAY) != 0) {
	(void) fprintf(stderr, "Fcntl failed on `/dev/console' (%s)\n", PERROR);
	return;
    }
#endif
#endif
    if (ioctl(ttyfd, TCSETAF, &tio) == -1) {
	(void) fprintf(stderr, "Ioctl failed on `/dev/console' (%s)\n", PERROR);
	return;
    }
    (void) close(ttyfd);
#endif
}

enum input_Status input_Poll(key, delay)
enum input_Key *key;
struct timeval *delay;
{
    enum input_Status result;

    static struct { /* has to be static */
	int type;	/* should be 6 or 10 if nothing happened */
	int position;	/* position code */
	int scancode;	/* key scan code */
	int status;	/* status bits */
    } key_event;

#ifndef GSLBUG
    int err, wait;

    plot_Flush(); /* make sure all output is visible */
    wait = 0;
    if ((err = gsevwt_(&wait, &key_event)) != 0) {
	(void) fprintf(plotfd, "Get event failed (gsevwt %s)\n", 
	    gsl_error(err));
	return(input_NONE);
    }
    else switch (key_event.type) {
    case 6:
	break;
    case 10:
	return(input_NONE);
    default:
	(void) fprintf(plotfd, "Received unexpected event %d (gsevwt)\n", 
	    key_event.type);
	return(input_NONE);
    }
#else
#ifdef notdef
    /*
     * select works only every second
     */
    static SELLIST(1, 1) rd, wr, ex;
    plot_Flush(); /* make sure all output is visible */

    rd.fdsmask[0] = 1 << ttyfd;
    select(NFDS(ttyfd + 1), &rd, &wr, &ex, delay);
#else
    plot_Flush(); /* make sure all output is visible */
    nap(delay);
#endif
    if (ioctl(ttyfd, TIONREAD, &nr) == -1) {
	(void) fprintf(stderr, "Ioctl failed on `/dev/console' (%s)\n", PERROR);
	return input_NONE;
    }
    if (nr < 7) 
	return(input_NONE);

    nr = read(ttyfd, inbuf, 7);
    if ( inbuf[0] != '\033' ) { /* ESC */ 
#ifdef DEBUG
	(void) fprintf(plotfd, "First char not escape\n");
#endif
	return(input_NONE);
    }
    if ( inbuf[1] != '[' )  { /* [   */ 
#ifdef DEBUG
	(void) fprintf(plotfd, "Second char not bracket\n");
#endif
	return(input_NONE);
    }
    switch ( inbuf[2] ) {
    case 'w' :
	/*
	 * keyboard
	 */
	key_event.position   = inbuf[3];
	key_event.scancode   = inbuf[4];
	key_event.status     = (inbuf[5] & 0xff) | (inbuf[6] << 8);
#ifdef DEBUG
	(void) fprintf(plotfd, "pos = %d, scancode %d status %x\n",
	     key_event.position, key_event.scancode, key_event.status);
#endif 
	break;
    case 'y' :
	/*
	 * mouse or tablet or dials
	 */
#ifdef DEBUG
	(void) fprintf(plotfd, "Mouse or tablet or dials\n");
#endif
	return(input_NONE);
    default :
#ifdef DEBUG
	(void) fprintf(plotfd, "Unknown event\n");
#endif
	return(input_NONE);
    }
#endif


    switch (key_event.position) {

      case 58: /* Control */
        *key = key_ROTLEFT;
        break;

      case 17: /* Q */
        *key = key_QUIT;
        break;

      case 60: /* Left Alt */
        *key = key_ROTRIGHT;
        break;

      case 43: /* Enter */
      case 108: /* Num-Enter */
      case 2: /* 1 */
      case 93: /* Num-1 */
        *key = key_START;
        break;

      case 3: /* 2 */
      case 98: /* Num-2 */
        *key = key_START_2;
        break;

      case 62: /* Right alt */
        *key = key_THRUST;
        break;

      case 26: /* p */
        *key = key_PAUSE;
        break;

      case 32: /* s */
        *key = key_TOGGLESOUND;
        break;

      case 57: /* Action */
      case 44: /* Action */
        *key = key_FIRE;
        break;

      case 61: /* space bar */
        *key = key_HYPERSPACE;
        break;
    }
    return (key_event.status  & 1 ? input_KEYDOWN : input_KEYUP);
}
/* hft_sound():
 *	Play sound with frequency duration and volume
 */
int 
/*ARGSUSED*/
hft_sound(hft, freq, dur, vol)
int hft;
short	freq, dur, vol;
{
    static struct    hfsound hfp;
    struct    hfintro *hp;

#ifdef DEBUG
    (void) fprintf(stderr, 
	"Sound f=%d d=%d v=%d\n", (int) freq, (int) dur, (int) vol);
#endif
#ifdef notdef
    conf.hf_op = HFCHGVOLUME;
    conf.hf_obj = (vol >> 14) << 24;
    if ( ioctl(ttyfd, HFRCONF, &conf) == -1 ) {
	(void) fprintf(stderr, 
	    "Ioctl failed on `/dev/console' (HFCHGVOLUME) (%s)\n", PERROR);
	return FALSE;
    }
#endif
    hp	    = (struct hfintro *) hfp.hf_intro;
    hp->hf_esc	= HFINTROESC;		    /* ESCape, '[' and 'x'  */
    hp->hf_lbr	= HFINTROLBR;		    /*	characters for VTD  */
    hp->hf_ex	= HFINTROEX;		    /*	prefix		    */
    hp->hf_len[0]   = 0;		    /* total number of	    */
    hp->hf_len[1]   = 0;		    /*	bytes in the	    */
    hp->hf_len[2]   = 0;		    /*	header less the 3   */
    hp->hf_len[3]   = sizeof(hfp) - 3;	    /*	in VTD prefix	    */
    hp->hf_typehi   = HFSOUNDCH;	    /* indicates type of    */ 
    hp->hf_typelo   = HFSOUNDCL;	    /*	data to virt termnl */ 
    hfp.hf_sublen   = 2;		    /* value always = 2	    */ 
    hfp.hf_subtype  = 1;		    /* value always = 1	    */
    hfp.hf_mode	    = HFEXECALWAYS;	    /* let's have sound !!! */
    hfp.hf_dur[0]   = (dur & 0xFF00) >> 8;  /* 0-high, 1-low dur bit*/
    hfp.hf_dur[1]   = (dur & 0x00FF);	    /*	grab from short dur */
    hfp.hf_freq[0]  = (freq & 0xFF00) >> 8; /* 0-hi, 1-lo freq bit  */
    hfp.hf_freq[1]  = (freq & 0x00FF);	    /*	grab from short freq*/

    write (ttyfd, &hfp, sizeof(hfp));	    /* send ASCII strg to VT*/
} /* end hft_sound */
/* nap():
 *	sleep...
 */
static int
nap(delay)
struct timeval *delay;
{
    struct timeval toend, now;
    gettimeofday(&toend, 0);
    toend.tv_usec += delay->tv_usec;
    toend.tv_sec += delay->tv_sec;
    while (toend.tv_usec >= 1000000) {
	toend.tv_usec -= 1000000;
	toend.tv_sec++;
    }
    do 
	gettimeofday(&now, 0);
    while(timercmp(&now, &toend, <));
} /* end nap */

