/*
 * Input driver for IBM RT Advanced Monochrome Display
 *
 * Note: Requires use of the kernel X driver because the RT keyboard
 * driver has been very badly implemented.
 */

#include <sys/time.h>
#include <machinecons/screen_conf.h>
#include <machinecons/xio.h>

#include "type.h"
#include "input.h"
#include "plot.h"

static XIoAddrAddr ioaddr;
static int fd;

boolean input_GlobalInit()
{
    int disc = E_XINPUT;

    fd = open("/dev/apa16", 2);
    if (fd < 0)
        return FALSE;

    ioctl(fd, EISETD, &disc);
    ioctl(fd, QIOCADDR, &ioaddr);

    return TRUE;
}

/*
 * Note:
 *
 *   Running Asteroids while in ITC's wm is deadly because wm
 *   barfs and locks up the console.
 *   I tried using EISETD with E_STDINPUT in this routine
 *   before exiting, but that turns out to have nothing
 *   to do with the problem.
 */

void input_Done()
{
    close(fd);
}

enum input_Status input_Poll(key, delay)
enum input_Key *key;
struct timeval *delay;
{
    enum input_Status result;
    XEvent *ev;
    int h;
    long mask = 1 << fd;

    plot_Flush(); /* make sure all output is visible */

    select(fd + 1, &mask, NULL, NULL, delay);

    if (ioaddr->ihead == ioaddr->itail)
        return input_NONE;

    ev = ioaddr->ibuff + ioaddr->ihead;

    switch (ev->xe_key) {
      case 0x11: /* Control */
        *key = key_ROTLEFT;
        break;

      case 0x15: /* Q */
        *key = key_QUIT;
        break;

      case 0x19: /* Left Alt */
        *key = key_ROTRIGHT;
        break;

#if 0   /* quick kluge around unkown bug */
      case 0x5a: /* Enter */
      case 0x79: /* Num-Enter */
#endif
      case 0x16: /* 1 */
      case 0x69: /* Num-1 */
        *key = key_START;
        break;

      case 0x1e: /* 2 */
      case 0x72: /* Num-2 */
        *key = key_START_2;
        break;

      case 0x39: /* Right alt */
        *key = key_THRUST;
        break;

      case 0x4d: /* p */
        *key = key_PAUSE;
        break;

      case 0x1b: /* s */
        *key = key_TOGGLESOUND;
        break;

      case 0x58: /* Action */
        *key = key_FIRE;
        break;

      case 0x29: /* space bar */
        *key = key_HYPERSPACE;
        break;
    }

    switch (ev->xe_type) {
        case XE_BUTTON:
            switch (ev->xe_device) {
#if 0
                case XE_MOUSE:
                case XE_TABLET:
                    switch (ev->xe_direction) {
                        case XE_KBTUP:
                            result = input_MOUSEUP;
                            break;
                        case XE_KBTDOWN:
                            result = input_MOUSEDOWN;
                            break;
                        default:
                            result = input_NONE;
                            break;
                    }
                    break;
#endif
                case XE_DKB:   /* Main keyboard */
                case XE_CONSOLE:
                    switch (ev->xe_direction) {
                        case XE_KBTUP:
                            result = input_KEYUP;
                            break;
                        case XE_KBTDOWN:
                            result = input_KEYDOWN;
                            break;
                        default:
                            result = input_NONE;
                            break;
                    }
                    break;
                default:
                    result = input_NONE;
                    break;
            }
            break;
#if 0
        case XE_MMOTION:
        case XE_TMOTION:
            result = input_MOUSEMOVE;
            break;
#endif
        default:
            result = input_NONE;
            break;
    }
            
    /* Now remove the event from the queue */

    h = ioaddr->ihead + 1;
    if (h > ioaddr->iqsize)
        h = 0;
    ioaddr->ihead = h;

    return result;
}

