#include <stdio.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#include <sys/types.h>
#include <sundev/kbio.h>

#include "type.h"
#include "input.h"
#include "plot.h"
#include "util.h"

static int fd;

boolean input_GlobalInit()
{
    int direct = 1;
    int trans = 0;

    fd = open("/dev/kbd", 2);
    if (fd < 0) {
	(void) fprintf(stderr, "Cannot open /dev/kbd (%s).\n", 
	    PERROR);
        return FALSE;
    }

    if (ioctl(fd, KIOCSDIRECT, &direct) == -1) {
	(void) fprintf(stderr, "Ioctl failed KIOCSDIRECT (%s).\n", 
	    PERROR);
        close(fd);
        return FALSE;
    }
    if (ioctl(fd, KIOCTRANS, &trans) == -1) {
	(void) fprintf(stderr, "Ioctl failed KIOCSTRANS (%s).\n", 
	    PERROR);
        close(fd);
        return FALSE;
    }
    if (fcntl(fd, F_SETFL, (FNDELAY|FASYNC)) == -1) {
	(void) fprintf(stderr, "Fcntl failed F_SETFL (%s).\n", 
	    PERROR);
        close(fd);
        return FALSE;
    }
    if (fcntl(fd, F_SETOWN, getpid()) == -1) {
	(void) fprintf(stderr, "Fcntl failed F_SETOWN (%s).\n", 
	    PERROR);
        close(fd);
        return FALSE;
    }

    return TRUE;
}

void input_Done()
{
    int direct = 1;
    int trans = 1;
    (void) ioctl(fd, KIOCSDIRECT, &direct);
    (void) ioctl(fd, KIOCTRANS, &trans);
    (void) close(fd);
}

enum input_Status input_Poll(key, delay)
enum input_Key *key;
struct timeval *delay;
{
    enum input_Status result;
    int h;
    long mask = 1 << fd;
    char c;

    plot_Flush(); /* make sure all output is visible */

    if (select(fd + 1, &mask, NULL, NULL, delay) <= 0)
        return input_NONE;

    read (fd, &c, 1);

    if (c == 127)
        return input_NONE;

#ifdef 0
    (void) printf(">>>%.2x\n", c & 0x7f);
#endif
    switch (c & 0x7f) {
#if defined(sparc) || defined(__sparc__)
      case 0x4c:   /* Control */
#else
      case 0x77:   /* Caps */
#endif
        *key = key_ROTLEFT;
        break;

#if defined(sparc) || defined(__sparc__)
      case 0x63:   /* Left Shift */
#else
      case 0x78:   /* Left */
#endif
        *key = key_ROTRIGHT;
        break;

#if defined(sparc) || defined(__sparc__)
      case 0x6e:   /*  Right Shift*/
#else
      case 0x7f:   /* Shift */
#endif
        *key = key_THRUST;
        break;

#if defined(sparc) || defined(__sparc__)
      case 0x59:   /* Return */
#else
      case 0x13:   /* Alternate */
#endif
        *key = key_FIRE;
        break;

      case 0x79:   /* Space */
        *key = key_HYPERSPACE;
        break;

      case 0x36:   /* q */
        *key = key_QUIT;
        break;

      case 0x1e:   /* 1 */
#if 0
      case 0x59:   /* Return */
#endif
        *key = key_START;
        break;

      case 0x1f:   /* 2 */
	*key = key_START_2;
	break;

      case 0x3f:   /* p */
        *key = key_PAUSE;
        break;

      case 0x4e:   /* s */
        *key = key_TOGGLESOUND;
        break;

      default:
        return input_NONE;
    }

    if (c & 0x80)
        return input_KEYUP;
    else
        return input_KEYDOWN;
}
