#include "linefont.h"

/*
 * Font character spaces are 7 by 11 units.  These
 * numbers are the appropriate char and line spacings.
 * Uppercase characters are drawn in a 6 by 8
 * area in the upper left of the character space.
 * Lowercase characters are drawn in a 6 by 10
 * area in the upper left, due to descenders.
 *
 * Descenders hang down an additional 2 units.
 * The font character is drawn with the left side
 * of the char at the specified x, and the baseline
 * at the specified y.
 *
 * Chars are Shapes which may be used as Sprite
 * shapes.  Thus chars may be scaled up, moving with,
 * a constant velocity, rotated or spinning, etc.
 */

#define E -999

VECT C_space[] = {
    {E,0}
};

VECT C_exclam[] = {
    {3,8},{3,2}, {3,0},{3,0}, {E,0}
};

VECT C_dquote[] = {
    {1,8},{1,6}, {5,8},{5,6}, {E,0}
};

VECT C_pound[] = {
    {1,8},{1,0}, {5,8},{5,0}, {0,6},{6,6}, {0,2},{6,2},
    {E,0}
};

VECT C_dollar[] = {
    {6,6},{5,7}, {5,7},{1,7}, {1,7},{0,6}, {0,6},{0,5},
    {0,5},{1,4}, {1,4},{5,4}, {5,4},{6,3}, {6,3},{6,2},
    {6,2},{5,1}, {5,1},{1,1}, {1,1},{0,2}, {2,8},{2,0},
    {4,8},{4,0}, {E,0}
};

VECT C_percent[] = {
    {1,1},{5,7}, {1,5},{2,6}, {4,2},{5,3}, {E,0}
};

VECT C_amper[] = {
    {1,0},{6,6}, {6,6},{6,7}, {6,7},{5,8}, {5,8},{4,8},
    {4,8},{3,7}, {3,7},{3,6}, {3,6},{6,3}, {6,3},{6,1},
    {6,1},{5,0}, {5,0},{3,0}, {3,0},{1,2}, {E,0}
};

VECT C_squote[] = {
    {3,8},{3,7}, {3,7},{2,6}, {E,0}
};

VECT C_lparen[] = {
    {4,8},{3,6}, {3,6},{3,2}, {3,2},{4,0}, {E,0}
};

VECT C_rparen[] = {
    {2,8},{3,6}, {3,6},{3,2}, {3,2},{2,0}, {E,0}
};

VECT C_aster[] = {
    {3,1},{3,7}, {1,6},{5,2}, {1,2},{5,6}, {E,0}
};

VECT C_plus[] = {
    {0,4},{6,4}, {3,1},{3,7}, {E,0}
};

VECT C_comma[] = {
    {2,1},{2,0}, {2,0},{1,-1}, {E,0}
};

VECT C_minus[] = {
    {0,4},{6,4}, {E,0}
};

VECT C_period[] = {
    {2,0},{2,0}, {E,0}
};

VECT C_slash[] = {
    {1,0},{5,8}, {E,0}
};

VECT C_0[] = {
    {0,1},{0,7}, {0,7},{1,8}, {1,8},{5,8}, {5,8},{6,7},
    {6,7},{6,1}, {6,1},{5,0}, {5,0},{1,0}, {1,0},{0,1},
    {0,0},{6,8}, {E,0}
};

VECT C_1[] = {
    {1,6},{3,8}, {3,8},{3,0}, {1,0},{5,0}, {E,0}
};

VECT C_2[] = {
    {0,7},{1,8}, {1,8},{5,8}, {5,8},{6,7}, {6,7},{6,5},
    {6,5},{0,1}, {0,1},{0,0}, {0,0},{6,0}, {E,0}
};

VECT C_3[] = {
    {0,7},{1,8}, {1,8},{5,8}, {5,8},{6,7}, {6,7},{6,5},
    {6,5},{5,4}, {5,4},{6,3}, {6,3},{6,1}, {6,1},{5,0},
    {5,0},{1,0}, {1,0},{0,1}, {1,4},{5,4}, {E,0}
};
    
VECT C_4[] = {
    {5,8},{5,0}, {1,8},{0,4}, {0,4},{6,4}, {E,0}
};
    
VECT C_5[] = {
    {6,8},{0,8}, {0,8},{0,4}, {0,4},{5,4}, {5,4},{6,3},
    {6,3},{6,1}, {6,1},{5,0}, {5,0},{1,0}, {1,0},{0,1},
    {E,0}
};
    
VECT C_6[] = {
    {6,7},{5,8}, {5,8},{1,8}, {1,8},{0,7}, {0,7},{0,1},
    {0,1},{1,0}, {1,0},{5,0}, {5,0},{6,1}, {6,1},{6,3},
    {6,3},{5,4}, {5,4},{0,4}, {E,0}
};

VECT C_7[] = {
    {0,7},{1,8}, {1,8},{6,8}, {6,8},{4,2}, {4,2},{4,0},
    {E,0}
};

VECT C_8[] = {
    {1,8},{5,8}, {5,8},{6,7}, {6,7},{6,5}, {6,5},{5,4},
    {5,4},{6,3}, {6,3},{6,1}, {6,1},{5,0}, {5,0},{1,0},
    {1,0},{0,1}, {0,1},{0,3}, {0,3},{1,4}, {1,4},{0,5},
    {0,5},{0,7}, {0,7},{1,8}, {1,4},{5,4}, {E,0}
};

VECT C_9[] = {
    {0,1},{1,0}, {1,0},{5,0}, {5,0},{6,1}, {6,1},{6,7},
    {6,7},{5,8}, {5,8},{1,8}, {1,8},{0,7}, {0,7},{0,5},
    {0,5},{1,4}, {1,4},{6,4}, {E,0}
};

VECT C_colon[] = {
    {3,6},{3,6}, {3,2},{3,2}, {E,0}
};

VECT C_semi[] = {
    {3,6},{3,6}, {3,3},{3,2}, {3,2},{2,1}, {E,0}
};

VECT C_less[] = {
    {5,8},{1,4}, {1,4},{5,0}, {E,0}
};

VECT C_equal[] = {
    {0,5},{6,5}, {0,3},{6,3}, {E,0}
};

VECT C_greater[] = {
    {1,8},{5,4}, {5,4},{1,0}, {E,0}
};

VECT C_ques[] = {
    {0,7},{1,8}, {1,8},{5,8}, {5,8},{6,7}, {6,7},{6,5},
    {6,5},{3,2}, {3,2},{3,1}, {3,0},{3,0}, {E,0}
};

VECT C_at[] = {
    {3,2},{4,3}, {4,3},{4,4}, {4,4},{3,5}, {3,5},{2,4},
    {2,4},{2,3}, {2,3},{3,2}, {3,2},{5,2}, {5,2},{6,3},
    {6,3},{6,7}, {6,7},{5,8}, {5,8},{1,8}, {1,8},{0,7},
    {0,7},{0,1}, {0,1},{1,0}, {1,0},{5,0}, {5,0},{6,1},
    {E,0}
};

VECT C_A[] = {
    {0,0},{0,6}, {0,6},{3,8}, {3,8},{6,6}, {6,6},{6,0},
    {0,4},{6,4},{E,0}
};

VECT C_B[] = {
    {0,0},{0,8}, {0,8},{5,8}, {5,8},{6,7}, {6,7},{6,5},
    {6,5},{5,4}, {5,4},{6,3}, {6,3},{6,1}, {6,1},{5,0},
    {5,0},{0,0}, {0,4},{5,4}, {E,0}
};

VECT C_C[] = {
    {6,7},{5,8}, {5,8},{1,8}, {1,8},{0,7}, {0,7},{0,1},
    {0,1},{1,0}, {1,0},{5,0}, {5,0},{6,1}, {E,0}
};

VECT C_D[] = {
    {0,0},{0,8}, {0,8},{5,8}, {5,8},{6,7}, {6,7},{6,1},
    {6,1},{5,0}, {5,0},{0,0}, {E,0}
};

VECT C_E[] = {
    {0,0},{6,0}, {0,4},{5,4}, {0,8},{6,8}, {0,0},{0,8},
    {E,0}
};

VECT C_F[] = {
    {0,4},{5,4}, {0,8},{6,8}, {0,0},{0,8}, {E,0}
};

VECT C_G[] = {
    {6,7},{5,8}, {5,8},{1,8}, {1,8},{0,7}, {0,7},{0,1},
    {0,1},{1,0}, {1,0},{5,0}, {5,0},{6,1}, {6,1},{6,4},
    {6,4},{3,4}, {E,0}
};

VECT C_H[] = {
    {0,0},{0,8}, {6,0},{6,8}, {0,4},{6,4}, {E,0}
};

VECT C_I[] = {
    {1,8},{5,8}, {1,0},{5,0}, {3,8},{3,0}, {E,0}
};

VECT C_J[] = {
    {4,8},{6,8}, {5,8},{5,1}, {5,1},{4,0}, {4,0},{1,0},
    {1,0},{0,1}, {E,0}
};

VECT C_K[] = {
    {0,0},{0,8}, {0,4},{6,8}, {0,4},{6,0}, {E,0}
};

VECT C_L[] = {
    {0,0},{0,8}, {0,0},{6,0}, {E,0}
};

VECT C_M[] = {
    {0,0},{0,8}, {0,8},{3,4}, {3,4},{6,8}, {6,8},{6,0},
    {E,0}
};

VECT C_N[] = {
    {0,0},{0,8}, {0,8},{6,0}, {6,0},{6,8}, {E,0}
};

VECT C_O[] = {
    {0,1},{0,7}, {0,7},{1,8}, {1,8},{5,8}, {5,8},{6,7},
    {6,7},{6,1}, {6,1},{5,0}, {5,0},{1,0}, {1,0},{0,1},
    {E,0}
};

VECT C_P[] = {
    {0,0},{0,8}, {0,8},{5,8}, {5,8},{6,7}, {6,7},{6,5},
    {6,5},{5,4}, {5,4},{0,4}, {E,0}
};

VECT C_Q[] = {
    {0,1},{0,7}, {0,7},{1,8}, {1,8},{5,8}, {5,8},{6,7},
    {6,7},{6,1}, {6,1},{5,0}, {5,0},{1,0}, {1,0},{0,1},
    {6,0},{3,3}, {E,0}
};

VECT C_R[] = {
    {0,0},{0,8}, {0,8},{5,8}, {5,8},{6,7}, {6,7},{6,5},
    {6,5},{5,4}, {5,4},{0,4}, {3,4},{6,0}, {E,0}
};

VECT C_S[] = {
    {6,7},{5,8}, {5,8},{1,8}, {1,8},{0,7}, {0,7},{0,5},
    {0,5},{1,4}, {1,4},{5,4}, {5,4},{6,3}, {6,3},{6,1},
    {6,1},{5,0}, {5,0},{1,0}, {1,0},{0,1}, {E,0}
};

VECT C_T[] = {
    {0,8},{6,8}, {3,8},{3,0}, {E,0}
};

VECT C_U[] = {
    {0,8},{0,1}, {0,1},{1,0}, {1,0},{5,0}, {5,0},{6,1},
    {6,1},{6,8}, {E,0}
};

VECT C_V[] = {
    {0,8},{3,0}, {3,0},{6,8}, {E,0}
};

VECT C_W[] = {
    {0,8},{0,0}, {0,0},{3,4}, {3,4},{6,0}, {6,0},{6,8},
    {E,0}
};

VECT C_X[] = {
    {0,0},{6,8}, {0,8},{6,0}, {E,0}
};

VECT C_Y[] = {
    {0,8},{0,6}, {0,6},{3,4}, {3,4},{6,6}, {6,6},{6,8},
    {3,4},{3,0}, {E,0}
};

VECT C_Z[] = {
    {0,8},{6,8}, {6,8},{0,0}, {0,0},{6,0}, {E,0}
};

VECT C_lbrack[] = {
    {4,8},{2,8}, {2,8},{2,0}, {2,0},{4,0}, {E,0}
};

VECT C_bslash[] = {
    {1,8}, {5,0}, {E,0}
};

VECT C_rbrack[] = {
    {2,8},{4,8}, {4,8},{4,0}, {4,0},{2,0}, {E,0}
};

VECT C_caret[] = {
    {0,5},{3,8}, {3,8},{6,5}, {E,0}
};

VECT C_uscore[] = {
    {0,0},{6,0}, {E,0}
};

VECT C_bquote[] = {
    {3,8},{3,7}, {3,7},{4,6}, {E,0}
};

VECT C_a[] = {
    {E,0}
};

VECT C_b[] = {
    {E,0}
};

VECT C_c[] = {
    {E,0}
};

VECT C_d[] = {
    {E,0}
};

VECT C_e[] = {
    {E,0}
};

VECT C_f[] = {
    {E,0}
};

VECT C_g[] = {
    {E,0}
};

VECT C_h[] = {
    {E,0}
};

VECT C_i[] = {
    {E,0}
};

VECT C_j[] = {
    {E,0}
};

VECT C_k[] = {
    {E,0}
};

VECT C_l[] = {
    {E,0}
};

VECT C_m[] = {
    {E,0}
};

VECT C_n[] = {
    {E,0}
};

VECT C_o[] = {
    {E,0}
};

VECT C_p[] = {
    {E,0}
};

VECT C_q[] = {
    {E,0}
};

VECT C_r[] = {
    {E,0}
};

VECT C_s[] = {
    {E,0}
};

VECT C_t[] = {
    {E,0}
};

VECT C_u[] = {
    {E,0}
};

VECT C_v[] = {
    {E,0}
};

VECT C_w[] = {
    {E,0}
};

VECT C_x[] = {
    {E,0}
};

VECT C_y[] = {
    {E,0}
};

VECT C_z[] = {
    {E,0}
};

VECT C_lbrace[] = {
    {4,8},{3,7}, {3,7},{3,5}, {3,5},{2,4}, {2,4},{3,3},
    {3,3},{3,1}, {3,1},{4,0}, {E,0}
};

VECT C_vbar[] = {
    {3,8},{3,0}, {E,0}
};

VECT C_rbrace[] = {
    {2,8},{3,7}, {3,7},{3,5}, {3,5},{4,4}, {4,4},{3,3},
    {3,3},{3,1}, {3,1},{2,0}, {E,0}
};

VECT C_tilde[] = {
    {0,4},{1,5}, {1,5},{2,5}, {2,5},{4,3}, {4,3},{5,3},
    {5,3},{6,4}, {E,0}
};

VECT C_block[] = {
    {0,0},{0,8}, {1,0},{1,8}, {2,0},{2,8}, {3,0},{3,8},
    {4,0},{4,8}, {5,0},{5,8}, {6,0},{6,8}, {0,0},{6,0},
    {0,1},{6,1}, {0,2},{6,2}, {0,3},{6,3}, {0,4},{6,4},
    {0,5},{6,5}, {0,6},{6,6}, {0,7},{6,7}, {0,8},{6,8},
    {E,0}
};

VECT *linechar[128] = {
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    C_space, C_exclam, C_dquote, C_pound,
    C_dollar, C_percent, C_amper, C_squote,
    C_lparen, C_rparen, C_aster, C_plus,
    C_comma, C_minus, C_period, C_slash,
    C_0, C_1, C_2, C_3, C_4, C_5, C_6, C_7,
    C_8, C_9, C_colon, C_semi,
    C_less, C_equal, C_greater, C_ques,
    C_at, C_A, C_B, C_C, C_D, C_E, C_F, C_G,
    C_H, C_I, C_J, C_K, C_L, C_M, C_N, C_O,
    C_P, C_Q, C_R, C_S, C_T, C_U, C_V, C_W,
    C_X, C_Y, C_Z, C_lbrack,
    C_bslash, C_rbrack, C_caret, C_uscore,
    C_bquote, C_a, C_b, C_c, C_d, C_e, C_f, C_g,
    C_h, C_i, C_j, C_k, C_l, C_m, C_n, C_o,
    C_p, C_q, C_r, C_s, C_t, C_u, C_v, C_w,
    C_x, C_y, C_z, C_lbrace,
    C_vbar, C_rbrace, C_tilde, C_block
};

static SHAPE *GenShape(ch, scale, ang)
long ch;
FRAC *scale;
long ang;
{
    int i;
    SHAPE *s;

    if (linechar[ch] == NULL)
        return NULL;

    s = shape_Create();

    for (i = 0; vect_GetX(&linechar[ch][i]) != E; i += 2) {
        LINE l;
        VECT from, to;
        vect_Copy(&from, &linechar[ch][i]);
        vect_Copy(&to, &linechar[ch][i + 1]);
        vect_SetY(&from, -vect_GetY(&from));
        vect_SetY(&to, -vect_GetY(&to));
        line_SetFT(&l, &from, &to);
        shape_Insert(s, &l);
    }

    shape_Scale(s, scale);
    shape_Rotate(s, ang);

    return s;
}

private void ComputeBBox(lf)
LINEFONT *lf;
{
    VECT cur, *max = &lf->maxSize;
    long count;
    SHAPE **sp = lf->chars;

    vect_SetXY(max, 0, 0);

    for (count = 128; count--; sp++)
        if (*sp != NULL) {
            shape_GetBoxSize(*sp, &cur);
            if (vect_GetX(&cur) > vect_GetX(max))
                vect_SetX(max, vect_GetX(&cur));
            if (vect_GetY(&cur) > vect_GetY(max))
                vect_SetY(max, vect_GetY(&cur));
        }
}

private void ComputeSpacings(lf)
LINEFONT *lf;
{
    FRAC f;
    VECT *charSpacing = &lf->charSpacing,
         *lineSpacing = &lf->lineSpacing;
    frac_SetND(&f, 1, 5);   /* Spacing as fraction of font size */

    vect_Copy(charSpacing, &lf->maxSize);
    vect_Scale(charSpacing, &f);
    vect_Add(charSpacing, &lf->maxSize);
    vect_SetY(charSpacing, 0);

    vect_Copy(lineSpacing, &lf->maxSize);
    vect_Scale(lineSpacing, &f);
    vect_Add(lineSpacing, &lf->maxSize);
    vect_SetX(lineSpacing, 0);
}

LINEFONT *linefont_Create(scale, ang)
FRAC *scale;
long ang;
{
    long ch;
    LINEFONT *lf = New(LINEFONT);

    for (ch = 0; ch < 128; ch++)
        lf->chars[ch] = GenShape(ch, scale, ang);

    ComputeBBox(lf);
    ComputeSpacings(lf);

    return lf;
}

void linefont_Destroy(lf)
LINEFONT *lf;
{
    long ch;

    for (ch = 0; ch < 128; ch++)
        if (lf->chars[ch] != NULL)
            shape_Destroy(lf->chars[ch]);
    Zap(lf);
}

SHAPE *linefont_Get(lf, ch)
LINEFONT *lf;
long ch;
{
    if (ch < 0 || ch > 128)
        return NULL;
    if (ch >= 'a' && ch <= 'z')
        ch -= 'a' - 'A';
    return lf->chars[ch];
}
