/*
 * Asteroids
 *
 *   by Curt McDowell
 *      Miles Bader
 *      William Lott
 */

#include <stdio.h>
#include <signal.h>
#include <pwd.h>
#include <string.h>
extern struct passwd *getpwuid();

#undef PROFILE

#include "asteroids.h"
#include "type.h"
#include "score.h"
#include "linefont.h"
#include "game.h"
#include "plot.h"
#include "rand.h"
#include "sound.h"
#include "high.h"
#include "sys.h"

#ifdef PROFILE

#ifdef sun
#define BASE 0x8000
#endif
#ifdef vax
#define BASE 2
#endif
#ifdef ibm032
#define BASE 0
#endif

#include <a.out.h>
#include <sys/param.h>
#include <sys/gprof.h>
#include <sys/file.h>

#define SIZE ((int) &etext - BASE)
static boolean profiling;
static char *profBuf;
static struct phdr header;
extern char etext;

static void StartProf() {
    profBuf = malloc(SIZE);
    bzero(profBuf, SIZE);
    header.lpc = (char *) BASE;
    header.hpc = header.lpc + SIZE;
    header.ncnt = SIZE + sizeof (header);
    profil(profBuf, SIZE, BASE, 0x10000L);
}

static void StopProf() {
    int fd;
    profil(0, 0, 0, 0);
    fd = open("gmon.out", O_CREAT | O_TRUNC | O_WRONLY, 0666);
    write(fd, &header, sizeof (header));
    write(fd, profBuf, SIZE);
    close(fd);
    free(profBuf);
}
#endif PROFILE

#define Dot() (printf("."), fflush(stdout))

#define MAXGAMES 20

static void Welcome()
{
    printf("\nWelcome to Asteroids [$Revision 2.1$]\n\n");
    printf("By:\n");
    printf("  Curt McDowell\n");
    printf("  Miles Bader\n");
    printf("  William Lott\n");
    printf("  Christos Zoulas\n\n");

    printf("**** NEW **** Control Keys:\n\n");
#ifdef RT
    printf("    Rotate Left        Ctrl\n");
    printf("    Rotate Right       Left Alt\n\n");
    printf("    Thrust             Right Alt\n");
    printf("    Fire               Action\n");
#endif
#ifdef SUN
# if defined(sparc) || defined(__sparc__)
    printf("    Rotate Left        Control\n");
    printf("    Rotate Right       Left Shift\n\n");
    printf("    Thrust             Right Shift\n");
    printf("    Fire               Return\n");
# else
    printf("    Rotate Left        Caps\n");
    printf("    Rotate Right       Left\n\n");
    printf("    Thrust             Right\n");
    printf("    Fire               Alternate\n");
# endif
#endif
#ifdef HP
    printf("    Rotate Left        Z\n");
    printf("    Rotate Right       X\n\n");
    printf("    Thrust             Shift\n");
    printf("    Fire               Return\n");
#endif
    printf("    Hyperspace         Space Bar or H\n\n");
    printf("    Quit               Q\n");
    printf("    Pause              P\n");
    printf("    Toggle Sound       S\n\n");
}

void cleanup(i)
int i;
{
    char *sig;
    input_Done();
    plot_Done();
    sound_Done();
    switch(i) {
    case SIGINT:  sig = "Interrupt"; break;
    case SIGQUIT: sig = "Quit"; break;
    case SIGHUP:  sig = "Hangup"; break;
    case SIGILL:  sig = "Illegal Instruction"; break;
#ifdef SIGBUS
    case SIGBUS:  sig = "Bus Error"; break;
#endif /* SIGBUS */
    case SIGSEGV: sig = "Segmention Violation"; break;
    case SIGFPE:  sig = "Floating Point Exception"; break;
    case SIGTERM: sig = "Terminated"; break;
    default:      sig = "Internal Error"; break;
    }
    if (i != 0) {
	(void) fprintf(stderr, "Asteroids received %s.\n",
	    sig);
	exit(sig);
    }
#ifdef SUN
    (void) fprintf(stderr, "%c", 'L' - 'A' + 1);
#endif
    (void) fprintf(stderr, "Bye\n");
    exit(0);
}

main(argc, argv)
int argc;
char **argv;
{
    SCORE *high;
    LINEFONT *font;
    int players;
    extern char *ttyname();

#ifdef PROFILE
    profiling = FALSE;
#endif PROFILE

#ifndef XWINDOWS
    /*
     * Start only from console, if name is not a.out
     */
    if (strcmp(*argv, "a.out") != 0 && 
	strcmp(ttyname(fileno(stderr)), "/dev/console") != 0) {
	(void) fprintf(stderr, "Can start asteroids only from the console.\n");
	exit(1);
    }
#endif

    argv++;
    argc--;
    while (argc>0) {
#ifdef PROFILE
        if (strcmp(*argv, "-p") == 0) {
            profiling = TRUE;
            StartProf();
        } else if (strcmp(*argv, "-f") == 0 && argc > 1) {
            argv++;
            argc--;
            /* Do something with argv[0] */
        } else
#endif PROFILE
	    if(strncmp(*argv,"-b",2)==0){
		char *p=(*argv+2);
		if(*p=='\0'){
		    ship_GlobalSetBounce(TRUE);
		    asteroids_GlobalSetBounce(TRUE);
		    missile_GlobalSetBounce(TRUE);
		}else
		    while(*p!='\0')
			switch(*p++){
			    case 'm':
				missile_GlobalSetBounce(TRUE); break;
			    case 's':
				ship_GlobalSetBounce(TRUE); break;
			    case 'a':
				asteroids_GlobalSetBounce(TRUE); break;
			}
	    }

        argv++;
        argc--;
    }

    Welcome();

#ifdef PROFILE
    if (profiling) {
        printf("*** Profiling Enabled\n");
        printf("*** Output will be stored in gmon.out\n\n");
    }
#endif PROFILE

    printf("Initializing -- please wait");

    rand_Seed();
    Dot();
    sound_GlobalInit();
    shape_GlobalInit();
    Dot();
    missile_GlobalInit();
    Dot();
    asteroids_GlobalInit();
    Dot();
    ship_GlobalInit();
    Dot();
    debris_GlobalInit();
    alien_GlobalInit();
    Dot();
    printf("\n\n");

    printf("Move mouse pointer into this window and press Return to start: ");
    while (getchar() != '\n')
        ;

    if (! plot_Init()) {
	fprintf(stderr, "Couldn't init plotting device.\n");
	exit(-1);
    }

    game_GlobalInit();

    if (! input_GlobalInit()) {
        fprintf(stderr, "Couldn't init input device.\n");
	plot_Done();
        exit(-1);
    }



    {
        VECT fontSize, scoreSize, screenSize, scorePos;

        high = score_Create();
        vect_SetXY(&fontSize, 2, 1);
	font=linefont_Create(&fontSize, 0);
        score_SetFont(high, font);
        score_GetSize(high, &scoreSize);
        plot_GetSize(&screenSize);
        vect_SetXY(&scorePos,HIGHX,(SCOREBARH - vect_GetY(&scoreSize)) / 2);
        score_SetPos(high, &scorePos);
        score_On(high);
    }

    plot_SetMode(plot_RES);
    plot_Clear();

    signal(SIGINT,cleanup);
    signal(SIGQUIT,cleanup);
    signal(SIGHUP,cleanup);
    signal(SIGILL,cleanup);
#ifdef SIGBUS
    signal(SIGBUS,cleanup);
#endif /* SIGBUS */
    signal(SIGSEGV,cleanup);
    signal(SIGTERM,cleanup);
    signal(SIGFPE,cleanup);

    do {
	GAME *games[MAXGAMES];
	int activePlayers,i;

        score_Update(high);

	players=game_Attract();

	for(i=0; i<players; i++)
	    games[i]=game_Create(i,1);

	activePlayers=players;
	while(activePlayers>0){
	    for(i=0; i<players; i++){
		if(games[i]!=NULL && !game_Turn(games[i])){
                    char *getenv();
		    SCORE *score=game_GetScore(games[i]);
                    char *name;

		    if (score_Get(score) > score_Get(high))
			score_Set(high, score_Get(score));

                    name = getenv("ASTNICK");
                    if (name == NULL)
                        name = getenv("USER");
		    if (name == NULL)
			name = getenv("LOGNAME");
		    if (name == NULL) {
			struct passwd *pwd;
			static char namebuf[sizeof(pwd->pw_name) + 1];
			if ((pwd = getpwuid(getuid())) == (struct passwd *) 0) 
			    name = "anon";
			else {
			    name = strncpy(namebuf, pwd->pw_name, 
					   sizeof(pwd->pw_name));
			    name[sizeof(pwd->pw_name)] = '\0';
			}
		    }
                    high_Record(name, score_Get(score));

		    game_Destroy(games[i]);
		    games[i]=NULL;

		    activePlayers--;
		}
	    }
	}
    } while (players>0);

    score_Destroy(high);
    linefont_Destroy(font);

#ifdef PROFILE
    if (profiling)
        StopProf();
#endif PROFILE
    cleanup(0);

}
