#include "type.h"
#include "mlist.h"

MLIST *mlist_Create()
{
    MLIST *ml=New(MLIST);

    ml->free=NULL;
    ml->active=NULL;
    ml->on=FALSE;
    ml->count=0;

    return ml;
}

void mlist_Destroy(ml)
MLIST *ml;
{
    struct mlElement *me,*next;

    for(me=ml->free; me!=NULL; me=next){
	next=me->next;
	missile_Destroy(me->missile);
	Zap(me);
    }

    for(me=ml->active; me!=NULL; me=next){
	next=me->next;
	missile_Destroy(me->missile);
	Zap(me);
    }

    Zap(ml);
}

static struct mlElement *NewME(ml)
MLIST *ml;
{
    struct mlElement *free=ml->free;

    if(free==NULL)
	free=New(struct mlElement);
    else
	ml->free=free->next;

    free->next=ml->active;
    free->prev=NULL;

    if(ml->active!=NULL)
	ml->active->prev=free;
    ml->active=free;

    free->missile=missile_Create();

    ml->count++;

    return free;
}

static void FreeME(ml,me)
MLIST *ml;
struct mlElement *me;
{
    if(me->prev==NULL)
	ml->active=me->next;
    else
	me->prev->next=me->next;

    if(me->next!=NULL)
	me->next->prev=me->prev;

    ml->count--;

    me->next=ml->free;
    ml->free=me;
}

void mlist_Update(ml)
MLIST *ml;
{
    struct mlElement *me,*next;

    for(me=ml->active; me!=NULL; me=next){
	next=me->next;
	missile_Update(me->missile);
	if(!missile_IsOn(me->missile))
	   FreeME(ml,me);
    }
}

void mlist_Fire(ml,pos,vel,velMul,timeout,radius)
MLIST *ml;
VECT *pos,*vel;
FRAC *velMul;
int timeout;
int radius;
{
    struct mlElement *me=NewME(ml);
    VECT v;

    /* Advance missile outside radius of the firing object */

    vect_Copy(&v, vel);
    vect_Scale(&v, velMul);

    if (vect_GetX(&v) != 0 || vect_GetY(&v) != 0) {
        VECT dist;
        register int radsqr = radius * radius;

        vect_SetXY(&dist, 0, 0);
        while (vect_SumSqr(&dist) <= radsqr)
            vect_Add(&dist, &v);
        vect_Add(pos, &dist);
        vect_Sub(pos, &v);     /* Make up for sprite not being on yet */
    }

    missile_SetPos(me->missile,pos);
    missile_SetVel(me->missile,vel);
    if(velMul!=NULL)
        missile_SetVelMul(me->missile,velMul);
    missile_SetTimeout(me->missile,timeout);
    if(ml->on)
        missile_On(me->missile);
    else
        missile_Off(me->missile);
}

void mlist_Enumerate(ml,proc,rock)
MLIST *ml;
int (*proc)();
char *rock;
{
    struct mlElement *me;

    for(me=ml->active; me!=NULL; me=me->next)
	if(!(*proc)(me->missile,rock))
	    break;
}

void mlist_On(ml)
MLIST *ml;
{
    struct mlElement *me;

    ml->on=TRUE;

    for(me=ml->active; me!=NULL; me=me->next)
	missile_On(me->missile);
}

void mlist_Off(ml)
MLIST *ml;
{
    struct mlElement *me;

    ml->on=FALSE;

    for(me=ml->active; me!=NULL; me=me->next)
	missile_Off(me->missile);
}
