
#include <stdio.h>
#include <gslerrno.h>
#include <signal.h>
#include "plot.h"
#include "util.h"
int plot_curX, plot_curY;
int plot_Width, plot_Height;
int three = 3, four = 4, six = 6;
FILE *plotfd;

#define GSLBUG
char *gsl_error();
static int buffer[128];
boolean plot_Init() 
{
    int ssize, fildes;
    int err, save_restore;
    int grant, retract;
    static struct gsl_dev {
	int id;
	int width;
	int height;
	int mm_width;
	int mm_height;
	int pits_per_pixel;
	int flags;
	int red_bits;
	int green_bits;
	int blue_bits;
	int min_curs_width;
	int min_curs_height;
	int max_curs_width;
	int max_curs_height;
	int color_table_size;
	int font_class;
	int logop;
    } dev;

    /*
     * Use gsl for maximum portability
     */
#ifdef notdef
    /*
     * Report all errors to /dev/console
     */
    if (freopen("/dev/console", "w", plotfd) == NIL(FILE)) {
	(void) fprintf(plotfd, "Could not reopen `/dev/console' (%s)\n", 
		PERROR);
	cleanup(-1);
    }
#endif
#ifdef GSLBUG
    ssize = 128; /* input ring buffer */
#else
    ssize = 0;
#endif
    plotfd = fopen("gslerr.out", "w"); /* send errors to plotfd */
    fildes = -1;
    save_restore = 1;
    grant = retract = 0;
    if ((err = gsinit_(buffer, &ssize, &save_restore, &grant, &retract, &fildes)) != 0) {
	(void) fprintf(plotfd, "Cannot initialize screen (gsinit %s)\n", 
	     gsl_error(err));
	return(FALSE);
    }
    gsqdsp_(&dev);
    switch (dev.id) {
    case 0x402 :
	(void) fprintf(plotfd, 
	    "IBM RT/PC Advanced Monochrome Graphics Display Adapter\n");
	break;
    case 0x405 :
	(void) fprintf(plotfd, 
	    "IBM RT/PC Extended Monochrome Graphics Display Adapter\n");
	break;
    case 0x406 :
	(void) fprintf(plotfd, 
	    "IBM Advanced Color Graphics Display Adapter\n");
	break;
    case 0x408 :
	(void) fprintf(plotfd, 
	    "IBM 5081 Display Adapter\n");
	break;
    default :
	(void) fprintf(plotfd, "Unknown display type %x\n", dev.id);
	break;
    }
    plot_Width = dev.width;
    plot_Height = dev.height;
    /*
     * Check it the adapter supports set reset and xor
     */
    if (( dev.logop & ((1 << 3) | (1 << 4) | (1 << 6)) ) !=
        ((1 << 3) | (1 << 4) | (1 << 6))) {
	(void) fprintf(plotfd, 
	    "Adapter does not handle the required logops.\n");
	return(TRUE);
    }

    return(TRUE);
}
void plot_Done()
{
    gsterm_(); /* Terminate gsl */
}

char *gsl_error(eno)
int eno;
{
    switch (eno) {
    case GS_SUCC :
	return("Successful");
    case GS_HBUS :
	return("Can't access hardware bus");
    case GS_ADPT :
	return("Invalid adaptor type");
    case GS_FONT :
	return("Can't access default font");
    case GS_RING :
	return("Ring buffer too small");
    case GS_INAC :
	return("Do not currently have screen");
    case GS_CORD :
	return("Invalid coordinate(s)");
    case GS_NCOR :
	return("Invalid num of coordinates");
    case GS_FBUF :
	return("Frame buffer overflow");
    case GS_UCUR :
	return("Undefined cursor");
    case GS_TABL :
	return("Invalid table length");
    case GS_COLI :
	return("Invalid color index");
    case GS_SYLI :
	return("Invalid style index or User line style undefined");
    case GS_CPID :
	return("Invalid code page identifier");
    case GS_BASL :
	return("Invalid baseline direction");
    case GS_FNTI :
	return("Invalid font index");
    case GS_FNTN :
	return("Invalid font file name");
    case GS_CURS :
	return("Invalid cursor size");
    case GS_CURO :
	return("Invalid cursor origin");
    case GS_CURV :
	return("Cursor not visible");
    case GS_USUC :
	return("Unsuccessful");
    case GS_VALG :
	return("Invalid valuator granularity");
    case GS_EVNT :
	return("Invalid event type");
    case GS_ROVR :
	return("Ring buffer overflow");
    case GS_UDRG :
	return("Ring undefined");
    case GS_PARM :
	return("Invalid number of esc parms");
    case GS_ICTL :
	return("Invalid final control char");
    case GS_NMEM :
	return("No memory available");
    case GS_LONS :
	return("Unsupported logical operator");
    case GS_PMSZ :
	return("Invalid Polymarker Size");
    case GS_PMOR :
	return("Invalid Polymarker Origin");
    case GS_PMSY :
	return("Invalid Polymarker Style");
    case GS_ULNS :
	return("Invalid User Line Style length or begin length");
    case GS_VGRN :
	return("Invalid Valuator Granularity");
    case GS_ELMM :
	return("Invalid Major/minor axis for Ellipses");
    case GS_RDUS :
	return("Invalid Radius for circles");
    case GS_ASTR :
	return("Invalid start pt for arcs");
    case GS_AEND :
	return("Invalid end point for arcs");
    case GS_ANGL :
	return("Invalid angle");
    case GS_NPLF :
	return("Incompatible data formats");
    case GS_HDLK :
	return("Hard Copy init rc");
    case GS_HDIM :
	return("Hard Copy init rc");
    case GS_HDDB :
	return("Hard Copy init rc");
    case GS_HDNA :
	return("Hard Copy init rc");
    case GS_HDMG :
	return("Hard Copy init rc");
    case GS_HDIF :
	return("Hard Copy init rc");
    case GS_HDSF :
	return("Hard Copy init rc");
    case GS_HDGO :
	return("Hard Copy init rc");
    case GS_HDGN :
	return("Hard Copy init rc");
    case GS_HDGU :
	return("Hard Copy init rc");
    case GS_IWID :
	return("Invalid bitblt width param");
    case GS_IHEI :
	return("Invalid bitblt height parm");
    case GS_IBPP :
	return("gsxblt  error: source bits per pixel invalid");
    case GS_IVWD :
	return("Invalid Pick Window size");
    case GS_PKAT :
	return("Pick already active");
    case GS_PREC :
	return("Invalid Precision value");
    case GS_EXPN :
	return("Invalid Expansion Factor");
    case GS_SPAC :
	return("Invalid Spacing value");
    case GS_XFRM :
	return("Invalid Transform mode");
    case GS_IVBF :
	return("Invalid buffer address");
    case GS_IVCT :
	return("Invalid geom font select");
    case GS_CEXP :
	return(
"Attempted a color expansion blt, but the dest pixmap was not a display adapter"
	);
    case GS_HIGH :
	return("Invalid Char height Geom txt");
    case GS_UPVT :
	return("Invalid Up vector	Geom txt");
    case GS_ALGN :
	return("Invalid Char Align	Geom txt");
    case GS_NOFT :
	return("No font loaded	Geom txt");
    case GS_OUTF :
	return("Inv output data format");
    case GS_INPF :
	return("Inv input data format");
    case GS_BMAX :
	return("bits per pixel > 8");
    case GS_PWID :
	return("pixmap width not a multiple of 16 bits.");
    case GS_FFMT :
	return("Invalid RTX font format");
    case GS_BCNT :
	return("too many fonts specified");
    case GS_BRED :
	return("failed reading in font file");
    case GS_NDMA :
	return("not enough DMA for fonts");
    case GS_MFLD :
	return("multiple fonts already loaded");
    case GS_BUFS :
	return("bad buffer size for send cmd");
    default:
	return("Unknown error");
    }
} /* end gsl_error */


