/*
 * Sun compatibility package
 * Uses sun as generic bitmap with Ras package.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/file.h>
#include <sys/mman.h>
#include <sun/fbio.h>
#include "type.h"
#include "vect.h"
#include "plot.h"
#include "util.h"

#define BM_MSBLEFT
#define BM_WORD u_short
/* #define BM_BLACK_ON_WHITE */

#include "plot.bytemap.c"

private int fd;
private char *allocMem;
private int fbsize;
private int videostate;
private int offset;

#define ROUNDUP(x, y) \
    (((int)(x) + (int)(y) - 1) & ~((int)(y) - 1))


boolean plot_Init()
{
    int bytesize, pgsize;
    int flag;
    struct fbtype fbtype;
    u_char *framebuf;

    if (plot_dpy != NULL)
        return FALSE;

    /*
     * If it is a damned cg4 we want to open only the color!
     */
    if ((fd = open("/dev/cgfour0", O_RDWR)) == -1)
	fd = open("/dev/fb", O_RDWR);
    if (fd < 0) {
	(void) fprintf(stderr, "Could not open `%s' (%s).\n", 
	    "/dev/fb", PERROR);
        return FALSE;
    }

    if (ioctl(fd, FBIOGTYPE, &fbtype) < 0) {
	(void) fprintf(stderr, "Ioctl failed FBIOGTYPE (%s).\n", 
	    PERROR);
        close(fd);
        return FALSE;
    }

    if (fbtype.fb_type != FBTYPE_SUN2COLOR &&
         fbtype.fb_type != FBTYPE_SUN3COLOR &&
	 fbtype.fb_type != FBTYPE_SUN4COLOR) {
	(void) fprintf(stderr, "No color framebuffer.\n");
        close(fd);
        return FALSE;
    }

#if defined(sparc) || defined(__sparc__)
    offset = (1024 * 128 * 2) * (fbtype.fb_type == FBTYPE_SUN4COLOR);
#else
    offset = 0;
#endif
    fbsize = fbtype.fb_size + offset;

#ifdef _MAP_NEW
    /*
     * New mmap calls
     */
    framebuf = (u_char *) mmap((caddr_t *) 0, fbsize, PROT_WRITE | PROT_READ, 
				 MAP_SHARED | _MAP_NEW, fd, 0);
    if ((int) framebuf == -1) {
	(void) fprintf(stderr, "Mmap failed (%s).\n", 
	    PERROR);
        close(fd);
        return FALSE;
    }
#else
    /*
     * Old frame buffer mmap calls
     */
# ifdef notdef
    /*
     * Use valloc here, it is easier
     */
    bytesize = (fbtype.fb_width * fbtype.fb_height) >> 3;
    pgsize = getpagesize();
    fbsize = ROUNDUP(fbsize, pgsize);

    allocMem = malloc(fbsize + pgsize);
    if (allocMem == NULL) {
        close(fd);
        return FALSE;
    }

    framebuf = (u_char *) ROUNDUP(allocMem, pgsize);
# else
    framebuf = (u_char *) valloc(fbsize);
    if (framebuf == NULL) {
        close(fd);
        return FALSE;
    }
# endif

    if (mmap(framebuf, fbsize, PROT_WRITE, MAP_SHARED, fd, 0) < 0) {
        close(fd);
        return FALSE;
    }

#endif
#ifdef 0
printf("type = %d width %d height %d size %d\n", fbtype.fb_type, fbtype.fb_width, fbtype.fb_height, fbsize);
#endif
    framebuf += offset;
    plot_dpy = bytemap_Display(framebuf, fbtype.fb_width, fbtype.fb_height);
    plot_Mode = plot_SET;

    flag = FBVIDEO_ON;
    if (ioctl(fd, FBIOSVIDEO, &flag) < 0) {
	(void) fprintf(stderr, "Ioctl failed FBIOSVIDEO (%s).\n", 
	    PERROR);
        close(fd);
        return FALSE;
    }
    return TRUE;
}

void plot_Done()
{
    u_char *fbaddr;
    if (plot_dpy != NULL) {
	fbaddr = (u_char *) bytemap_Bytes(plot_dpy);
	fbaddr -= offset;
        munmap(fbaddr, fbsize);
        ioctl(fd, TIOCNOTTY, 0);
        close(fd);
        Zap(plot_dpy);
        plot_dpy = NULL;
    }
}
