#include "plot.h"
#include "ship.h"
#include "shape.h"
#include "vect.h"
#include "control.h"
#include "rand.h"
#include "soundeff.h"

#ifdef SUN
#define VELUNIT 113
#endif

#ifndef VELMULT
#define VELMULT 250
#endif

#ifndef VELUNIT
#define VELUNIT 100
#endif

private VECT ShipShape[] = {
    {-4, 4},
    {-2, 1},
    {-2, -1},
    {-4, -4},
    {8, 0}
};

private VECT FlameShape[] = {
    {-2, 1},
    {-2, -1},
    {-6, 0}
};

/* if true, ships bounce off edge, otherwise they wrap */
static boolean bounce=FALSE;

void ship_GlobalSetBounce(b)
boolean b;
{
    bounce=b;
}

private VECT Acl[SHIPROTATIONS];

private SHAPE *Ships[SHIPROTATIONS];
private SHAPE *Flames[SHIPROTATIONS];

void ship_GlobalInit()
{
    int rot;
    VECT v;
    int angle;

    Ships[0] = shape_CreatePolygon(ShipShape,
      sizeof(ShipShape) / sizeof(ShipShape[0]));
    Flames[0] = shape_CreatePolygon(FlameShape,
      sizeof(FlameShape) / sizeof(FlameShape[0]));

    vect_SetXY(&v, 5, 2);
    shape_Scale(Ships[0], &v);
    shape_Scale(Flames[0], &v);

    vect_SetXY(&Acl[0], VELUNIT, 0);

    for (rot = 1; rot < SHIPROTATIONS; rot++) {
	angle = 360 * rot / SHIPROTATIONS;
        Ships[rot] = shape_Create();
        shape_Copy(Ships[rot], Ships[0]);
        shape_Rotate(Ships[rot], angle);

	Flames[rot] = shape_Create();
	shape_Copy(Flames[rot], Flames[0]);
	shape_Rotate(Flames[rot], angle);

        vect_SetXY(&Acl[rot], VELUNIT, 0);
        cordic_Rotate(&Acl[rot], angle);
    }
}

void ship_Die(s)
SHIP *s;
{
    VECT pos,vel;
    FRAC scale;

    sprite_Off(s->sprite);
    sprite_Off(s->flames);

    ship_GetPos(s,&pos);
    ship_GetVel(s,&vel);
    ship_GetVelMul(s,&scale);
    vect_Scale(&vel,&scale);
    vect_Add(&pos,&vel); /* Compensate for pause */
    
    debris_SetPos(s->debris,&pos);
    debris_SetVel(s->debris,&vel);
    debris_Start(s->debris);

    s->thrusting=FALSE;
    s->boom=TRUE;
}

SHIP *ship_Create()
{
    VECT screen;
    SHIP *s = New(SHIP);
    FRAC f;

    plot_GetSize(&screen);

    s->sprite = sprite_Create();
    sprite_SetShapes(s->sprite, Ships, SHIPROTATIONS);
    sprite_SetBounds(s->sprite,
      1, vect_GetX(&screen) - 2, SCOREBARH + 1, vect_GetY(&screen) - 2);

    s->flames = sprite_Create();
    sprite_SetShapes(s->flames, Flames, SHIPROTATIONS);
    sprite_SetBounds(s->flames,
      1, vect_GetX(&screen) - 2, SCOREBARH + 1, vect_GetY(&screen) - 2);

    s->missiles=mlist_Create();
    s->debris=debris_Create(0,0,0);

    if(bounce) {
	sprite_SetBounce(s->sprite,TRUE);
	sprite_SetBounce(s->flames,TRUE);
    }
    else {
	sprite_SetWrap(s->sprite,TRUE);
	sprite_SetWrap(s->flames,TRUE);
    }


    frac_SetND(&f, 1, VELMULT);
    sprite_SetVelMul(s->sprite, &f);
    sprite_SetVelMul(s->flames, &f);

    s->hyperspace = 0;
    s->thrusting = FALSE;
    s->rotleft = s->rotright = 0;
    s->boom=FALSE;

    return s;
}

void ship_Destroy(s)
SHIP *s;
{
    mlist_Destroy(s->missiles);
    debris_Destroy(s->debris);
    sprite_Destroy(s->sprite);
    sprite_Destroy(s->flames);

    Zap(s);
}

void ship_Start(s)
SHIP *s;
{
    VECT v;

    plot_GetSize(&v);
    vect_DivConst(&v, 2);
    sprite_SetPos(s->sprite, &v);
    sprite_SetPos(s->flames, &v);

    vect_SetXY(&v, 0, 0);
    sprite_SetVel(s->sprite, &v);
    sprite_SetVel(s->flames, &v);
    sprite_SetInc(s->sprite, 0);
    sprite_SetInc(s->flames, 0);
    sprite_SetIndex(s->sprite, 0);
    sprite_SetIndex(s->flames, 0);

    s->hyperspace = 0;
    s->thrusting = FALSE;
    s->rotleft = s->rotright = 0;
    s->boom=FALSE;

    ship_On(s);
}

void ship_Update(s)
SHIP *s;
{
    VECT v;

    mlist_Update(s->missiles);

    if(s->boom){
	debris_Update(s->debris);
	if(debris_IsClear(s->debris))
	    s->boom=FALSE;
    }
	   
    if(s->hyperspace>0){
	if (--s->hyperspace == 0) {
	    sprite_On(s->sprite);
	    s->thrusting = FALSE;
	    plot_GetSize(&v);
	    vect_SetXY(&v,
		       rand_Natural(vect_GetX(&v)),
		       rand_Natural(vect_GetY(&v)));
	    sprite_SetPos(s->sprite,&v);
	    vect_SetXY(&v,0,0);
	    sprite_SetVel(s->sprite,&v);
            sprite_SetInc(s->sprite,0);
	    sprite_On(s->sprite);
	    sprite_Off(s->flames);
	}
    }
    
    if (s->thrusting) {
        sprite_GetVel(s->sprite, &v);
        vect_Add(&v, &Acl[sprite_GetIndex(s->sprite)]);
        sprite_SetVel(s->sprite, &v);
	sprite_SetVel(s->flames,&v);
	sprite_GetPos(s->sprite,&v);
	sprite_SetPos(s->flames,&v);
	sprite_SetIndex(s->flames, sprite_GetIndex(s->sprite));
	soundeff_Thrust();
	sprite_On(s->flames);
    }
    else 
	sprite_Off(s->flames);

    sprite_Update(s->sprite);
    sprite_Update(s->flames);
}

void ship_On(s)
SHIP *s;
{
    sprite_On(s->sprite);
    sprite_Off(s->flames);
    mlist_On(s->missiles);
    s->boom=FALSE;
}

void ship_Off(s)
SHIP *s;
{
    sprite_Off(s->sprite);
    sprite_Off(s->flames);
    mlist_Off(s->missiles);
    debris_Off(s->debris);
}

void ship_Command(s,cmd)
SHIP *s;
enum control_Command cmd;
{
    VECT pos, vel, mul;
    FRAC f;

    if(s->hyperspace>0)
	return;

    switch (cmd) {
        case control_THRUSTON:
            s->thrusting = TRUE;
            break;

        case control_THRUSTOFF:
            s->thrusting = FALSE;
            break;

        case control_ROTATELEFTON:
            sprite_SetInc(s->sprite, -1);
            s->rotleft = TRUE;
            break;

        case control_ROTATERIGHTON:
            sprite_SetInc(s->sprite, 1);
            s->rotright = TRUE;
            break;

        case control_ROTATELEFTOFF:
            sprite_SetInc(s->sprite, s->rotright ? 1 : 0);
            s->rotleft = FALSE;
            break;

        case control_ROTATERIGHTOFF:
            sprite_SetInc(s->sprite, s->rotleft ? -1 : 0);
            s->rotright = FALSE;
            break;

	case control_HYPERSPACE:
	    sprite_Off(s->sprite);
	    sprite_Off(s->flames);
            s->thrusting=FALSE;
	    s->hyperspace=HYPERCOUNT;
	    break;

	case control_FIRE:
            if (mlist_GetCount(s->missiles) >= MISSILELIMIT)
                break;

	    sprite_GetVel(s->sprite, &vel);
	    vect_Copy(&mul, &Acl[sprite_GetIndex(s->sprite)]);
	    vect_MulConst(&mul, SHIPSHOTSPEED);
	    vect_Add(&vel, &mul);
            frac_SetND(&f, 1, VELMULT);

	    sprite_GetPos(s->sprite, &pos);

            mlist_Fire(s->missiles, &pos, &vel, &f, SHIPSHOTRANGE,
              shape_GetRadius(ship_GetCurShape(s)));
            soundeff_ShipFire();

	    break;

	default:
            break;
    }
}
