#ifndef sound_h
#define sound_h

#include "type.h"

/*
 * Sound interface definition:
 *
 * void sound_Play(f, d, v) unsigned short f, d, v;
 *
 * Takes a volume in the range 0-65535
 *     which ranges from minimum to maximum over the full hardware range.
 * Takes a frequency in the range 0-65535
 *     in units of Hertz.  0 is counted as a rest.
 * Takes a duration in the range 0-65535
 *     in units of 1/128 second (I think)
 *
 * Never blocks, and sound commands are queued up.
 * If the sound is turned off, Play will not queue
 * any sound.  It is initially on.
 *
 * If the speaker is not successfully initialized,
 * GlobalInit returns FALSE and sound is permanently off.
 */

boolean sound_GlobalInit();
void sound_Done();

void sound_Play();

void sound_Off();
void sound_On();
boolean sound_IsOn();

#endif sound_h
