#include "sound.h"

#include <machineio/speakerio.h>

/*
 * If speakerfd is -1, the sound may not be turned on.
 */

private int speakerfd = -1;
private boolean isOn = FALSE;

boolean sound_GlobalInit()
{
    speakerfd = open("/dev/speaker", 1);
    if (speakerfd < 0) {
        speakerfd = -1;
        isOn = FALSE;
    } else
        isOn = TRUE;

    return isOn;
}

void sound_Done()
{
    if (speakerfd >= 0)
        close(speakerfd);
}

void sound_On()
{
    if (speakerfd >= 0)
        isOn = TRUE;
}

void sound_Off()
{
    isOn = FALSE;
}

void sound_Play(f, d, v)
unsigned short f, d, v;
{
    struct spk_blk sp;

    if (! isOn)
        return;

    sp.volume = (v >> 14);    /* RT volume is 0 to 3 */
    sp.duration = (d >> 1);   /* dur is 0 to 32767 */

    /* The RT speaker hardware is excessively obnoxious for freqs!! */

    if (f < 23) {
	sp.freqhigh = 0;
	sp.freqlow = SPKOLOMIN;
    } else if (f < 46) {
	sp.freqhigh = 64;
	sp.freqlow = (char) ((6000.0 /(float) f) - 9.31);
    } else if (f < 91) {
	sp.freqhigh = 32;
	sp.freqlow = (char) ((12000.0 /(float) f) - 9.37);
    } else if (f < 182) {
	sp.freqhigh = 16;
	sp.freqlow = (char) ((24000.0 /(float) f) - 9.48);
    } else if (f < 363) {
	sp.freqhigh = 8;
	sp.freqlow = (char) ((48000.0 /(float) f) - 9.71);
    } else if (f < 725) {
	sp.freqhigh = 4;
	sp.freqlow = (char) ((96000.0 /(float) f) - 10.18);
    } else if (f < 1433) {
	sp.freqhigh = 2;
	sp.freqlow = (char) ((192000.0 /(float) f) - 11.10);
    } else if (f < 12020) {
	sp.freqhigh = 1;
	sp.freqlow = (char) ((384000.0 /(float) f) - 12.95);
    } else {
	sp.freqhigh = 0;
	sp.freqlow = SPKOLOMIN;
    }

    write(speakerfd, (char *) &sp, sizeof (sp));
}
