#include "plot.h"
#include "sprite.h"

/*
 * Sprite Methods
 */

private void InitState(sp)
struct spritestate *sp;
{
    sp->nfigs = 0;
    sp->figs = NULL;
    sp->xl = sp->xh = sp->yl = sp->yh = 0;
    sp->wrap = FALSE;
    vect_SetXY(&sp->pos, 0, 0);
    sp->index = 0;

    sp->on = FALSE;
}

SPRITE *sprite_Create()
{
    SPRITE *s = New(SPRITE);

    InitState(&s->state);
    InitState(&s->oldState);

    vect_SetXY(&s->vel, 0, 0);
    frac_SetND(&s->vel_mul, 1, 1);
    s->index_inc = 0;

    return s;
}

void sprite_Destroy(s)
SPRITE *s;
{
    sprite_Off(s);
    sprite_Update(s);
    Zap(s);
}

void sprite_SetShapes(s, shs, n)
SPRITE *s;
SHAPE **shs;
int n;
{
    s->state.nfigs = n;
    s->state.figs = shs;
}

void sprite_Update(s)
SPRITE *s;
{
    long rad;
    VECT size, v;
    long w, h;
    long diff;
    long x,y,xh,yh,xl,yl;

    if (! s->state.on && ! s->oldState.on)
	return;

    /*
     * Update shape if necessary
     */

    s->state.index += s->index_inc;
    while (s->state.index < 0)
        s->state.index += s->state.nfigs;
    while (s->state.index >= s->state.nfigs)
        s->state.index -= s->state.nfigs;

    /*
     * Update position
     */

    vect_Copy(&v, &s->vel);
    vect_Scale(&v, &s->vel_mul);
    vect_Add(&s->state.pos, &v);

    rad = shape_GetRadius(s->state.figs[s->state.index]);

    plot_GetSize(&size);
    vect_GetXY(&size, &w, &h);

    xl=s->state.xl;
    xh=s->state.xh;

    x=vect_GetX(&s->state.pos);

    while((diff=xl-x+rad)>0)
	if(s->state.wrap)
	    x=xh-diff-rad;	/* - rad so it doesn't hit the OTHER edge */
	else if(s->state.bounce){
	    x=xl+diff+rad;
	    vect_SetX(&s->vel,-vect_GetX(&s->vel));
        }else{
            s->state.on = FALSE;
            break;
        }
    while((diff=x+rad-xh)>0)
	if(s->state.wrap)
	    x=xl+diff+rad;	/* + rad so it doesn't hit the OTHER edge */
	else if(s->state.bounce){
	    x=xh-diff-rad;
	    vect_SetX(&s->vel,-vect_GetX(&s->vel));
        }else{
            s->state.on = FALSE;
            break;
        }

    yl=s->state.yl;
    yh=s->state.yh;

    y=vect_GetY(&s->state.pos);

    while((diff=yl-y+rad)>0)
	if(s->state.wrap)
	    y=yh-diff-rad;	/* - rad so it doesn't hit the OTHER edge */
	else if(s->state.bounce){
	    y=yl+diff+rad;
	    vect_SetY(&s->vel,-vect_GetY(&s->vel));
        }else{
            s->state.on = FALSE;
            break;
        }
    while((diff=y+rad-yh)>0)
	if(s->state.wrap)
	    y=yl+diff+rad;	/* + rad so it doesn't hit the OTHER edge */
	else if(s->state.bounce){
	    y=yh-diff-rad;
	    vect_SetY(&s->vel,-vect_GetY(&s->vel));
        }else{
            s->state.on = FALSE;
            break;
        }

    vect_SetX(&s->state.pos,x);
    vect_SetY(&s->state.pos,y);

    if (s->oldState.on && s->oldState.nfigs > 0) {
        plot_SetMode(plot_RES);
        shape_Draw(s->oldState.figs[s->oldState.index], &s->oldState.pos);
    }

    if (s->state.on && s->state.nfigs > 0) {
        plot_SetMode(plot_SET);
        shape_Draw(s->state.figs[s->state.index], &s->state.pos);
    }

    s->oldState = s->state;
}
