#ifndef sprite_h
#define sprite_h

#include "type.h"
#include "shape.h"
#include "vect.h"

#define SPRITE struct sprite

struct spritestate {
    boolean on;

    int nfigs;
    SHAPE **figs;

    long xl, xh;
    long yl, yh;      /* Screen bounds for wrapping */

    boolean wrap;     /* should wrap around instead of going offscreen */
    boolean bounce;   /* should bounce off edge instead of going offscreen */

    VECT pos;
    int index;
};

struct sprite {
    struct spritestate state, oldState;

    /* d(Sprite State)/dt */

    VECT vel;
    FRAC vel_mul;
    int index_inc;
};

SPRITE *sprite_Create();
void sprite_Destroy();

void sprite_SetShapes();
#define sprite_On(s) ((s)->state.on = TRUE)
#define sprite_Off(s) ((s)->state.on = FALSE)
#define sprite_IsOn(s) ((s)->state.on)

#define sprite_GetShapes(s) ((s)->state.figs)
#define sprite_GetNumShapes(s) ((s)->state.nfigs)
#define sprite_GetShapeAt(s, ind) ((s)->state.figs[ind])
#define sprite_GetCurShape(s) ((s)->state.figs[(s)->state.index])
#define sprite_SetBounds(s, xlow, xhigh, ylow, yhigh) \
    ((s)->state.xl = (xlow), (s)->state.xh = (xhigh), \
     (s)->state.yl = (ylow), (s)->state.yh = (yhigh))
#define sprite_GetBounds(s, xlowp, xhighp, ylowp, yhighp) \
    (*(xlowp) = (s)->state.xl, *(xhighp) = (s)->state.xh, \
     *(ylowp) = (s)->state.yl, *(yhighp) = (s)->state.yh)
#define sprite_SetWrap(s, w) ((s)->state.wrap = (w))
#define sprite_SetBounce(s,b) ((s)->state.bounce=(b))
#define sprite_SetPos(s, p) vect_Copy(&(s)->state.pos, (p))
#define sprite_SetPosAndOld(s, p) \
    (vect_Copy(&(s)->state.pos, (p)), \
     vect_Copy(&(s)->oldState.pos, (p)))
#define sprite_GetPos(s, p) vect_Copy(p, &(s)->state.pos)
#define sprite_RefPos(s) (&(s)->state.pos)
#define sprite_RefOldPos(s) (&(s)->oldState.pos)
#define sprite_SetIndex(s, i) ((s)->state.index = (i))
#define sprite_GetIndex(s) ((s)->state.index)

#define sprite_SetVel(s, v) vect_Copy(&(s)->vel, (v))
#define sprite_GetVel(s, v) vect_Copy((v), &(s)->vel)
#define sprite_RefVel(s) (&(s)->vel)
#define sprite_SetVelMul(s, f) frac_Copy(&(s)->vel_mul, (f))
#define sprite_GetVelMul(s, f) frac_Copy((f), &(s)->vel_mul)
#define sprite_SetInc(s, i) ((s)->index_inc = (i))
#define sprite_GetInc(s) ((s)->index_inc)

void sprite_Update();

#endif sprite_h
